#!/usr/bin/python3
#coding:utf-8

# タブ型画像ビューア「ZetaView（Pil版）」：Version13


# Copyright 2017 天杉 善哉 (あますぎ ぜんざい Amasugi Zenzai) All rights reserved.

# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 

import wx
import sys
import re
import os
import copy
from PIL import ImageChops
from PIL import Image, ImageEnhance
import configparser
import wx.lib.scrolledpanel
import mimetypes

#↓これを先にやっておかないと、あとのクラス変数にwxイメージを使う段階でエラーになる。
application = wx.App()


def image_rotation( pil_image, rotation_angle, bg_color = ( 180, 180, 180 ), testmode = False ):
	#OpenCVのときのように背景色をlistで渡してしまった場合のために、tupleに変換しておく。
	bg_color = tuple( bg_color )

	#PILの回転は反時計回りのようなので、時計回りになるように角度を反転させておく
	rotation_angle = 360 - rotation_angle

	origin_height = pil_image.size[1]
	origin_width = pil_image.size[0]

	diagonal1 = ( origin_height ** 2 + origin_width ** 2 ) ** 0.5

	#ただの対角線の長さで背景を作ると、回転時に白い背景が混ざるので
	#その更なる対角線の√2より少し大きい1.6倍くらいで背景画像を作る。
	diagonal2 = int( diagonal1 * 1.6 )

	zero_point = [ int( ( diagonal2 - origin_width ) / 2  ), int( ( diagonal2 - origin_height ) / 2 ) ]

	#回転した本番画像をそのままautocropすると、万一にも画像の端が背景色と同じだった場合に正常にautocropできない。
	#なので、本番画像と同じサイズの単色画像（背景色の反転色）を用意して、同じように背景に貼り付け、回転した参考画像から切り抜き範囲を取得する。
	reference_image = Image.new( "RGB", ( origin_width, origin_height ), ( 255 - bg_color[0], 255 - bg_color[1], 255 - bg_color[2] ) )
	reference_bg = Image.new( "RGB", ( diagonal2, diagonal2 ), bg_color )

	reference_bg.paste( reference_image, ( zero_point[0], zero_point[1] ) )
	rotated_reference = reference_bg.rotate( rotation_angle )

	#回転によって現れた四隅の黒を除去する。
	#後でautocropするので、ギリギリではなく余白（適当に画像対角線の5%ほど、上限30pixel）をつけて切り取る。
	#正方形から正方形を切り抜くので、X軸Y軸で共用の最小・最大値を取得する。
	cut_margin = int( diagonal1 * 0.05  )
	if cut_margin > 30:
		cut_margin = 30
	widecut_min = int( ( diagonal2 - diagonal1 ) / 2 - cut_margin )
	widecut_max = int( ( diagonal2 - diagonal1 ) / 2 + diagonal1 + cut_margin )

	margin_reference = rotated_reference.crop( ( widecut_min, widecut_min, widecut_max, widecut_max ) )


	#オートクロップ
	empty_image = Image.new( "RGB", margin_reference.size, bg_color )
	diff = ImageChops.difference( margin_reference, empty_image )
	bbox = diff.getbbox()
	#テストモードなら、本番画像を回転するという重い処理をせずに、参考画像を切り抜いて返す
	if testmode is True:
		dummy_autocroped_image = margin_reference.crop(bbox)

		return dummy_autocroped_image

	else:
		#本番画像を、参考画像と同様に背景に貼り付けて回転させ、参考画像から得た切り抜き範囲で切り抜く
		background_image = Image.new( "RGB", ( diagonal2, diagonal2 ), bg_color )
		background_image.paste( pil_image, ( zero_point[0], zero_point[1] ) )

		rotated_pil_image = background_image.rotate( rotation_angle )
		margin_target = rotated_pil_image.crop( ( widecut_min, widecut_min, widecut_max, widecut_max ) )
		autocroped_image = margin_target.crop(bbox)

		return autocroped_image


#2つの画像が縦長なら横に並べ、そうでないなら引数の1つめの画像イメージのみを返す関数。
def picture_narrow2_or_broad1( pict_path_1, pict_path_2, right_or_left, borderline_rate = 1.0 , effects_dictionary = {}, bg_color = ( 180, 180, 180 ), testmode = False ):

	infomations = {}

	try:
		img1 = Image.open( pict_path_1 )
	except:
		img1 = Image.new( "RGB", ( 600, 300 ), (50,50,50) )
		reference_values.add_read_error( pict_path_1 )

	#貼付けできるかを試して、正常に読み込めているかを再度確認する
	try:
		Image.new( "RGB", img1.size ).paste( img1, ( 0, 0 ) )
	except:
		img1 = Image.new( "RGB", ( 600, 300 ), (50,50,50) )
		reference_values.add_read_error( pict_path_1 )

	filename1 = os.path.basename( pict_path_1 )

	#テストモードでは、回転以外のエフェクトは行わない（後述のリサイズもしない）
	if testmode is False:
		#明るさとコントラストの調整は回転処理の前に置く必要がある。
		#（非90°倍数の回転で生じる背景を変色させないため）
		if "brightness" in effects_dictionary:
			brightness_percent = effects_dictionary.get( "brightness" )
			img1 = ImageEnhance.Brightness( img1 ).enhance( brightness_percent / 100.0 )

		if "contrast" in effects_dictionary:
			contrast_percent = effects_dictionary.get( "contrast" )
			img1 = ImageEnhance.Contrast( img1 ).enhance( contrast_percent / 100.0 )

		if "horizontal-reverse" in effects_dictionary:
			img1 = img1.transpose( Image.FLIP_LEFT_RIGHT )

		if "vertical-reverse" in effects_dictionary:
			img1 = img1.transpose( Image.FLIP_TOP_BOTTOM )

	if "rotate" in effects_dictionary:
		img1 = image_rotation( img1, effects_dictionary.get( "rotate" ), bg_color, testmode )

	infomations['name1'] = filename1
	infomations['path1'] = pict_path_1
	infomations['width1'] = img1.size[0]
	infomations['height1'] = img1.size[1]
	infomations['resize-rate1'] = 1.0
	if right_or_left == "LeftToRight":
		infomations['direction1'] = "左"
	else:
		infomations['direction1'] = "右"

	#わざわざ別の変数に代入するのは、現在のPILと検討中のOpenCVでサイズ取得の順番が逆のため。
	#（PILのsizeは(横のサイズ、縦のサイズ)だがOpenCVでは(縦、横、色数)で返ってくる）
	img1_size_x = img1.size[0]
	img1_size_y = img1.size[1]

	#画像ファイルを縦横比次第で連結する
	#borderline_rateが1より小さいと、縦長の画像でないと連結しなくなる（正方形では単体表示になる）
	#逆に1より大きくなるほど、横長の画像でも2つを連結して表示するようになる。

	#最初は不等号の左辺が分数だったが、0による除算のエラーを避けるために分母を右辺に移動した。
	if pict_path_2 is not None and os.path.exists( pict_path_2 ) and os.path.isfile(pict_path_2 ) and 1.0 * img1_size_x < borderline_rate * img1_size_y:
		try:
			img2 = Image.open( pict_path_2 )
		except:
			img2 =  Image.new( "RGB", ( 200, 100 ), ( 50, 50, 50 ) )
			reference_values.add_read_error( pict_path_2 )

		#貼付けできるかを試して、正常に読み込めているかを再度確認する
		try:
			Image.new( "RGB", img2.size ).paste( img2, ( 0, 0 ) )
		except:
			img2 = Image.new( "RGB", ( 600, 300 ), (50,50,50) )
			reference_values.add_read_error( pict_path_2 )

		filename2 = os.path.basename( pict_path_2 )
		dummy_image = Image.new( "RGB", ( 4, 2 ), ( 0, 0, 0 ) )

		if testmode is False:
			if "brightness" in effects_dictionary:
				brightness_percent = effects_dictionary.get( "brightness" )
				img2 = ImageEnhance.Brightness( img2 ).enhance( brightness_percent / 100.0 )

			if "contrast" in effects_dictionary:
				contrast_percent = effects_dictionary.get( "contrast" )
				img2 = ImageEnhance.Contrast( img2 ).enhance( contrast_percent / 100.0 )

			if "horizontal-reverse" in effects_dictionary:
				img2 = img2.transpose( Image.FLIP_LEFT_RIGHT )

			if "vertical-reverse" in effects_dictionary:
				img2 = img2.transpose( Image.FLIP_TOP_BOTTOM )

		if "rotate" in effects_dictionary:
			img2 = image_rotation( img2, effects_dictionary.get( "rotate" ), bg_color, testmode )

		img2_size_x = img2.size[0]
		img2_size_y = img2.size[1]

		if 1.0 * img2_size_x < borderline_rate * img2_size_y:

			infomations['name2'] = filename2
			infomations['path2'] = pict_path_2
			infomations['width2'] = img2.size[0]
			infomations['height2'] = img2.size[1]
			infomations['resize-rate2'] = 1.0
			if right_or_left == "LeftToRight":
				infomations['direction2'] = "右"
			else:
				infomations['direction2'] = "左"

			if img1_size_y == img2_size_y:
				if testmode is True:
					img_united = dummy_image
				else:
					img_united = Image.new( "RGB", ( img1_size_x + img2_size_x ,img1_size_y ) )

				if right_or_left == "LeftToRight":
					if testmode is False:
						img_united.paste( img1, ( 0, 0 ) )
						img_united.paste( img2, ( img1_size_x, 0 ) )
					filenames = [ os.path.basename( pict_path_1 ), os.path.basename( pict_path_2 ) ]
				else:
					if testmode is False:
						img_united.paste( img2, ( 0, 0 ) )
						img_united.paste( img1, ( img2_size_x, 0 ) )
					filenames = [ os.path.basename( pict_path_2 ), os.path.basename( pict_path_1 ) ]

			elif img1_size_y < img2_size_y:
				if testmode is True:
					img_united = dummy_image
				else:
					#ANTIALIASをBICUBICにすると、少しぼやけた感じになる。BILINEARだとさらにぼやける。基本のNEARESTは粗すぎて論外。
					img1_large = img1.resize( ( int( img1_size_x * img2_size_y /img1_size_y ), img2_size_y ), Image.LANCZOS )
					img_united = Image.new( "RGB", ( img1_large.size[0] + img2_size_x, img2_size_y ) )

					if right_or_left == "LeftToRight":
						img_united.paste( img1_large, ( 0, 0 ) )
						img_united.paste( img2, ( img1_large.size[0], 0 ) )
						filenames = [ os.path.basename( pict_path_1 ), os.path.basename( pict_path_2 ) ]
					else:
						img_united.paste( img2, ( 0, 0 ) )
						img_united.paste( img1_large, ( img2_size_x, 0 ) )
						filenames = [ os.path.basename( pict_path_2 ), os.path.basename( pict_path_1 ) ]

				infomations['resize-rate1'] = 1.0 * img2_size_y / img1_size_y

			else:
				if testmode is True:
					img_united = dummy_image
				else:
					img2_large = img2.resize( ( int( img2_size_x * img1_size_y / img2_size_y ), img1_size_y ), Image.LANCZOS )
					img_united = Image.new( "RGB", ( img1_size_x + img2_large.size[0], img1_size_y ) )

					if right_or_left == "LeftToRight":
						img_united.paste( img1, ( 0, 0 ) )
						img_united.paste( img2_large, ( img1_size_x, 0 ) )
						filenames = [ os.path.basename( pict_path_1 ), os.path.basename( pict_path_2 ) ]
					else:
						img_united.paste( img2_large, ( 0, 0 ) )
						img_united.paste( img1, ( img2_large.size[0], 0 ) )
						filenames = [ os.path.basename( pict_path_2 ), os.path.basename( pict_path_1 ) ]

				infomations['resize-rate2'] = 1.0 * img1_size_y / img2_size_y

			return [ img_united, infomations ]

		else:
			return [ img1, infomations ]

	else:
		return [ img1, infomations ]


#ファイルサイズをGBまでの単位で返す（単位を加える都合上、intではなくstring。TB以上は未対応）
def get_filesize( filepath ):
	size_byte = os.path.getsize( filepath )
	if os.path.isdir( filepath ):
		return "directory"

	#最初は1KB=1024、1MB=1048576、1GB=1073741824でやろうとしたが
	#Nautilusのサイズ表示と食い違ってしまうので1000の倍数に変更した
	elif size_byte >= 1000000000:
		scaled_size = round( float( size_byte / float( 1000000000 ) ), 1 )
		return str( scaled_size ) + "GB"

	elif size_byte >= 1000000:
		scaled_size = round( float( size_byte / float( 1000000 ) ), 1 )
		return str( scaled_size ) + "MB"

	elif size_byte >= 1000:
		scaled_size = round( float( size_byte / float( 1000 ) ), 1 )
		return str( scaled_size ) + "KB"

	else:
		return str( scaled_size ) + "B"


#ファイル名を「半角数字を大小で比較しながら」ソートする関数。
#Pythonでの自然順ソートの方法がわからなかったので、これを作った。
def maybe_filelist_natsort( filelist ):
	double_list = []
	sorted_list = []

	for filename in filelist:
		expanded_name = ""
		zeropool = ""

		#python2系まではunicode扱いにならないので変換する
		if sys.version_info[0] < 3:
			temporary = filename.decode( 'utf-8' )
		else:
			temporary = filename

		for i in range( len( temporary ) ):
			#半角数字なら、半角数字が続く限り蓄積しておく
			if temporary[i] in [ "0", "1", "2", "3", "4", "5", "6", "7", "8", "9" ]:
				zeropool += temporary[i]
				#最後の字なら、0を足して数字を結合して蓄積を初期化
				if i == len( temporary ) - 1:
					expanded_name += "0" * ( 255 - len( zeropool ) )
					expanded_name += zeropool
					zeropool = ""

			#半角数字以外なら、そのまま結合
			#その前に、半角数字が蓄積していれば0を足して結合
			else:
				#半角数字が蓄積していれば0を足して結合して蓄積を初期化
				if zeropool != "":
					expanded_name += "0" * ( 255 - len( zeropool ) )
					expanded_name += zeropool
					zeropool = ""

				expanded_name += temporary[i]

		double_list.append( [ expanded_name, filename ] )

	double_list.sort()
	sorted_list = [ x[1] for x in double_list ]

	return sorted_list


#フォルダ直下の画像ファイルのリストを返す関数
def detect_imagefiles_under_folder( directory ):
	imagefile_list = []

	if os.path.exists( directory ) and os.path.isdir( directory ):
		for i in maybe_filelist_natsort( os.listdir( directory ) ):
			if os.path.isfile( os.path.join ( directory, i ) ) :
				#画像のmimetypeになっているファイルだけをリストに追加する
				if re.match( "image/", str( mimetypes.guess_type( os.path.join ( directory, i ) )[0] ) ):
					imagefile_list.append( os.path.join ( directory, i ) )

		return imagefile_list

	else:
		return False


#ドラッグ＆ドロップの処理
class FileDrop( wx.FileDropTarget ):

	def __init__( self, window ):
		wx.FileDropTarget.__init__( self )
		self.window = window

	def OnDropFiles( self, x, y, dropfiles ):

		if reference_values.get_drop_mode() == "current-right":
			self.open_in_current_right( dropfiles )

		elif reference_values.get_drop_mode() == "ask" and self.window.notebook.GetSelection() < self.window.notebook.GetPageCount() - 1:
			ask_dialog = DropAskDialog()

			if ask_dialog.ShowModal() == wx.ID_OK:
				open_choice = ask_dialog.get_list_choice()

				if open_choice == 1:
					self.open_in_current_right( dropfiles )
				else:
					self.open_in_rightend( dropfiles )

			ask_dialog.Destroy()

		else:
			self.open_in_rightend( dropfiles )

		return True

	def open_in_rightend( self, detected_files ):
		for filepath in detected_files:
			self.window.file_open( filepath )

	def open_in_current_right( self, detected_files ):
		for filepath in detected_files:
			self.window.file_open( filepath, self.window.notebook.GetSelection() + 1 )


class DropAskDialog( wx.Dialog ):

	def __init__( self ):
		wx.Dialog.__init__( self, None, -1, "ファイルを開く場所の選択", size=( 300, 200 ) )

		element_array = ( "右端のタブに開く", "現在のタブの右に開く" )
		self.listbox = wx.ListBox( self, wx.ID_ANY, choices=element_array, style=wx.LB_SINGLE)

		self.ok_button = wx.Button( self, wx.ID_OK, "OK" )
		self.ok_button.SetDefault()
		self.cancel_button = wx.Button( self, wx.ID_CANCEL, "Cancel" )
		twin_buttons_sizer = wx.BoxSizer( wx.HORIZONTAL )
		twin_buttons_sizer.Add( self.ok_button )
		twin_buttons_sizer.Add( self.cancel_button )

		sizer = wx.BoxSizer( wx.VERTICAL )
		sizer.Add( self.listbox, 1, wx.ALL | wx.EXPAND, 10 )
		sizer.Add( twin_buttons_sizer, 0, wx.ALIGN_CENTER )
		self.SetSizer(sizer)

	def get_list_choice( self ):
		return self.listbox.GetSelection()


#拡張Frame
#ステータスバーの表示・非表示の切り替えやファイルオープン処理、縦横比境界値と
#背景色のおよび背景画像などを追加している
class extended_frame( wx.Frame ):

	def __init__( self, *args, **kwargs ):
		wx.Frame.__init__( self, *args, **kwargs )

		self.notebook = extended_notebook(self, wx.ID_ANY)
		self.notebook.SetBackgroundColour('#CCCCCC')
		self.notebook.SetForegroundColour('#303030')

		sizer = wx.BoxSizer( wx.VERTICAL )
		sizer.Add( self.notebook, 1, wx.ALL | wx.EXPAND, 0 )
		self.SetSizer( sizer )

		#ステータスバーを用意しておくが、隠しておく（オプションでTrue指定されれば関数で後からONにする）
		self.statusbar = self.CreateStatusBar()
		self.statusbar.Hide()
		#トグル用に、現在ステータスバーを表示しているかどうかを管理する変数
		self.statusbar_status = False

		#ドロップの設定
		drop_target = FileDrop( self )
		self.SetDropTarget( drop_target )

		#タブの中・大ジャンプの移動量
		self.tab_middle_jump_distance = 10
		self.tab_large_jump_distance = 30
		#画像の中・大ジャンプの移動量
		self.image_middle_jump_distance = 10
		self.image_large_jump_distance = 30


		##以下、ショートカットキーの設定

		#キー操作のID
		id_left = wx.NewIdRef().GetId()
		id_shift_left = wx.NewIdRef().GetId()
		id_alt_left = wx.NewIdRef().GetId()

		id_right = wx.NewIdRef().GetId()
		id_shift_right = wx.NewIdRef().GetId()
		id_alt_right = wx.NewIdRef().GetId()

		id_up = wx.NewIdRef().GetId()
		id_pageup = wx.NewIdRef().GetId()
		id_shift_up = wx.NewIdRef().GetId()
		id_shift_pageup = wx.NewIdRef().GetId()
		id_alt_up = wx.NewIdRef().GetId()
		id_alt_pageup = wx.NewIdRef().GetId()
		id_down = wx.NewIdRef().GetId()
		id_pagedown = wx.NewIdRef().GetId()
		id_shift_down = wx.NewIdRef().GetId()
		id_shift_pagedown = wx.NewIdRef().GetId()
		id_alt_down = wx.NewIdRef().GetId()
		id_alt_pagedown = wx.NewIdRef().GetId()

		id_ctrl_up = wx.NewIdRef().GetId()
		id_ctrl_down = wx.NewIdRef().GetId()
		id_ctrl_pageup = wx.NewIdRef().GetId()
		id_ctrl_pagedown = wx.NewIdRef().GetId()

		id_ctrl_w = wx.NewIdRef().GetId()
		id_ctrl_d = wx.NewIdRef().GetId()
		id_alt_s = wx.NewIdRef().GetId()
		id_f5 = wx.NewIdRef().GetId()

		id_home = wx.NewIdRef().GetId()
		id_end = wx.NewIdRef().GetId()
		id_ctrl_home = wx.NewIdRef().GetId()
		id_ctrl_end = wx.NewIdRef().GetId()

		id_f11 = wx.NewIdRef().GetId()

		#キーイベントとIDを関数にバインド
		self.Bind( wx.EVT_MENU, self.hotkey_left, id = id_left )
		self.Bind( wx.EVT_MENU, self.hotkey_shift_left, id = id_shift_left )
		self.Bind( wx.EVT_MENU, self.hotkey_alt_left, id = id_alt_left )

		self.Bind( wx.EVT_MENU, self.hotkey_right, id = id_right )
		self.Bind( wx.EVT_MENU, self.hotkey_shift_right, id = id_shift_right )
		self.Bind( wx.EVT_MENU, self.hotkey_alt_right, id = id_alt_right )

		self.Bind( wx.EVT_MENU, self.hotkey_up, id = id_up )
		self.Bind( wx.EVT_MENU, self.hotkey_pageup, id = id_pageup )
		self.Bind( wx.EVT_MENU, self.hotkey_shift_up, id = id_shift_up )
		self.Bind( wx.EVT_MENU, self.hotkey_shift_pageup, id = id_shift_pageup )
		self.Bind( wx.EVT_MENU, self.hotkey_alt_up, id = id_alt_up )
		self.Bind( wx.EVT_MENU, self.hotkey_alt_pageup, id = id_alt_pageup )
		self.Bind( wx.EVT_MENU, self.hotkey_down, id = id_down )
		self.Bind( wx.EVT_MENU, self.hotkey_pagedown, id = id_pagedown )
		self.Bind( wx.EVT_MENU, self.hotkey_shift_down, id = id_shift_down )
		self.Bind( wx.EVT_MENU, self.hotkey_shift_pagedown, id = id_shift_pagedown )
		self.Bind( wx.EVT_MENU, self.hotkey_alt_down, id = id_alt_down )
		self.Bind( wx.EVT_MENU, self.hotkey_alt_pagedown, id = id_alt_pagedown )

		self.Bind( wx.EVT_MENU, self.hotkey_ctrl_up, id = id_ctrl_up )
		self.Bind( wx.EVT_MENU, self.hotkey_ctrl_down, id = id_ctrl_down )
		self.Bind( wx.EVT_MENU, self.hotkey_ctrl_pageup, id = id_ctrl_pageup )
		self.Bind( wx.EVT_MENU, self.hotkey_ctrl_pagedown, id = id_ctrl_pagedown )

		self.Bind( wx.EVT_MENU, self.hotkey_ctrl_w, id = id_ctrl_w )
		self.Bind( wx.EVT_MENU, self.hotkey_ctrl_d, id = id_ctrl_d )
		self.Bind( wx.EVT_MENU, self.hotkey_alt_s, id = id_alt_s )
		self.Bind( wx.EVT_MENU, self.hotkey_f5, id = id_f5 )

		self.Bind( wx.EVT_MENU, self.hotkey_home, id = id_home )
		self.Bind( wx.EVT_MENU, self.hotkey_end, id = id_end )
		self.Bind( wx.EVT_MENU, self.hotkey_ctrl_home, id = id_ctrl_home )
		self.Bind( wx.EVT_MENU, self.hotkey_ctrl_end, id = id_ctrl_end )

		self.Bind( wx.EVT_MENU, self.hotkey_f11, id = id_f11 )

		self.hotkey_list = []
		self.hotkey_list.append( ( wx.ACCEL_NORMAL, wx.WXK_LEFT, id_left ) )
		self.hotkey_list.append( ( wx.ACCEL_SHIFT, wx.WXK_LEFT, id_shift_left ) )
		self.hotkey_list.append( ( wx.ACCEL_ALT, wx.WXK_LEFT, id_alt_left ) )
		self.hotkey_list.append( ( wx.ACCEL_NORMAL, wx.WXK_RIGHT, id_right ) )
		self.hotkey_list.append( ( wx.ACCEL_SHIFT, wx.WXK_RIGHT, id_shift_right ) )
		self.hotkey_list.append( ( wx.ACCEL_ALT, wx.WXK_RIGHT, id_alt_right ) )
		self.hotkey_list.append( ( wx.ACCEL_NORMAL, wx.WXK_UP, id_up ) )
		self.hotkey_list.append( ( wx.ACCEL_SHIFT, wx.WXK_UP, id_shift_up ) )
		self.hotkey_list.append( ( wx.ACCEL_ALT, wx.WXK_UP, id_alt_up ) )
		self.hotkey_list.append( ( wx.ACCEL_NORMAL, wx.WXK_DOWN, id_down ) )
		self.hotkey_list.append( ( wx.ACCEL_SHIFT, wx.WXK_DOWN, id_shift_down ) )
		self.hotkey_list.append( ( wx.ACCEL_ALT, wx.WXK_DOWN, id_alt_down ) )
		self.hotkey_list.append( ( wx.ACCEL_NORMAL, wx.WXK_PAGEUP, id_pageup ) )
		self.hotkey_list.append( ( wx.ACCEL_SHIFT, wx.WXK_PAGEUP, id_shift_pageup ) )
		self.hotkey_list.append( ( wx.ACCEL_ALT, wx.WXK_PAGEUP, id_alt_pageup ) )
		self.hotkey_list.append( ( wx.ACCEL_NORMAL, wx.WXK_PAGEDOWN, id_pagedown ) )
		self.hotkey_list.append( ( wx.ACCEL_SHIFT, wx.WXK_PAGEDOWN, id_shift_pagedown ) )
		self.hotkey_list.append( ( wx.ACCEL_ALT, wx.WXK_PAGEDOWN, id_alt_pagedown ) )
		self.hotkey_list.append( ( wx.ACCEL_CTRL, wx.WXK_UP, id_ctrl_up ) )
		self.hotkey_list.append( ( wx.ACCEL_CTRL, wx.WXK_DOWN, id_ctrl_down ) )
		self.hotkey_list.append( ( wx.ACCEL_CTRL, wx.WXK_PAGEUP, id_ctrl_pageup ) )
		self.hotkey_list.append( ( wx.ACCEL_CTRL, wx.WXK_PAGEDOWN, id_ctrl_pagedown ) )
		self.hotkey_list.append( ( wx.ACCEL_CTRL, ord( "w" ), id_ctrl_w ) )
		self.hotkey_list.append( ( wx.ACCEL_CTRL, ord( "d" ), id_ctrl_d ) )
		self.hotkey_list.append( ( wx.ACCEL_ALT, ord( "s" ), id_alt_s ) )
		self.hotkey_list.append( ( wx.ACCEL_NORMAL, wx.WXK_F5, id_f5 ) )
		self.hotkey_list.append( ( wx.ACCEL_NORMAL, wx.WXK_HOME, id_home ) )
		self.hotkey_list.append( ( wx.ACCEL_NORMAL, wx.WXK_END, id_end ) )
		self.hotkey_list.append( ( wx.ACCEL_CTRL, wx.WXK_HOME, id_ctrl_home ) )
		self.hotkey_list.append( ( wx.ACCEL_CTRL, wx.WXK_END, id_ctrl_end ) )
		self.hotkey_list.append( ( wx.ACCEL_NORMAL, wx.WXK_F11, id_f11 ) )

		self.SetAcceleratorTable( wx.AcceleratorTable( self.hotkey_list ) )


	#ステータスバーの表示・非表示を切り替える関数。通常はトグルだが、表示のみ、非表示のみも一応は引数次第で可能にしておく。
	def statusbar_show_or_hide( self, mode = "toggle" ):
		choice = ""
		if self.notebook.GetPageCount() >= 100:
			question_dialog = wx.MessageDialog( None, "※このダイアログはタブを100個以上開いている場合に現れます\n\nタブを大量に開いている時にステータスバーの表示・非表示を切り替えると、すべてのタブで表示領域の大きさが変わります。\nすべてのタブで一気に画像リサイズが始まるので\nしばらく応答しなくなります。\n\nそれでもステータスバー表示を切り替えますか？", "ステータスバー切り替えの前に", wx.YES_NO | wx.ICON_QUESTION )

			if question_dialog.ShowModal() == wx.ID_NO:
				choice = "NO"
			else:
				choice = "YES"

			question_dialog.Destroy()

		if choice != "NO":
			if mode == 'show':
				self.statusbar.Show()
				self.statusbar_status = True
			elif mode == 'hide':
				self.statusbar.Hide()
				self.statusbar_status = False
			else:
				if self.statusbar_status is True:
					self.statusbar.Hide()
					self.statusbar_status = False
				else:
					self.statusbar.Show()
					self.statusbar_status = True

	def fileopen_dialog( self, directory_path = "", tab_insert_position = -1 ):

		if directory_path == "":
			if reference_values.get_default_directory() == "":
				fdialog = wx.FileDialog( self, "画像ファイルか、画像のパスを記したテキストファイルを選択してください" )
			else:
				fdialog = wx.FileDialog( self, "画像ファイルか、画像のパスを記したテキストファイルを選択してください", reference_values.default_directory )
		else:
			fdialog = wx.FileDialog( self, "画像ファイルか、画像のパスを記したテキストファイルを選択してください", directory_path )

		if fdialog.ShowModal() == wx.ID_OK:
			filename = fdialog.GetFilename()
			dirpath = fdialog.GetDirectory()

			if sys.version_info[0] < 3:
				filename = filename.encode( 'utf-8' )
				dirpath = dirpath.encode( 'utf-8' )

			filepath = os.path.join ( dirpath, filename )

			self.file_open( filepath, tab_insert_position )

		fdialog.Destroy()
		self.notebook.SetFocus()


	def file_open( self, filepath, tab_insert_position = -1 ):

		#渡されたパスがフォルダなら、直下の画像1つだけを新しいタブで開く
		if os.path.isdir( filepath ):
			detected_imagefiles = detect_imagefiles_under_folder( filepath )

			if len( detected_imagefiles ) > 0:
				if tab_insert_position < 0:
					self.notebook.add_panel( detected_imagefiles[0] )
				else:
					self.notebook.add_panel( detected_imagefiles[0], tab_insert_position )

			else:
				error_dialog =  wx.MessageDialog( None, "フォルダ「" + filepath + "」にある画像を開こうとしましたが、フォルダ直下に画像ファイルが見つかりませんでした。", "Error", wx.OK | wx.ICON_ERROR )
				error_dialog.ShowModal()
				error_dialog.Destroy()

		#渡されたパスがファイルだったら、mimetypeに応じて処理する
		elif os.path.isfile( filepath ):
			#選択したファイルのmimetypeが画像なら、新しいタブで開く
			#空のファイルだとmimetypeがNoneになってしまうので、念の為にstr()で囲っておく
			if re.match( "image/", str( mimetypes.guess_type( filepath )[0] ) ):
				if tab_insert_position < 0:
					self.notebook.add_panel( filepath )
				else:
					self.notebook.add_panel( filepath, tab_insert_position )

			#それらの拡張子以外なら、画像ファイルのパスを記したリストファイルだとみなして読み込んでいく
			else:
				f = open( filepath, "r" )
				relatively_position = 0

				for one_line in f.readlines():
					#次の行で改行を除去しないとファイル名として正しく扱えない。
					one_line = one_line.rstrip( '\n' )
					one_line = one_line.rstrip( '\r' )
					line_abspath = os.path.abspath( one_line )

					if os.path.isfile( line_abspath ) and re.match( "image/", str( mimetypes.guess_type( line_abspath )[0] ) ):
						if tab_insert_position < 0:
							self.notebook.add_panel( line_abspath )
						else:
							self.notebook.add_panel( line_abspath, tab_insert_position + relatively_position )
							relatively_position += 1

				f.close()


	def add_picture_tab( self, picture_path_list = [] ):

		#リストが空（ファイルを引数で与えずに起動した場合）は、ファイル選択ダイアログを出す
		if picture_path_list == []:
			self.fileopen_dialog()

		else:
			for pic_file in picture_path_list:
				self.notebook.add_panel( pic_file )

	#（パネルからの要求で）画像を中・大規模移動するときの移動量を返す
	def get_image_jump_distance( self, scale = "" ):

		if scale == "middle":
			return self.image_middle_jump_distance
		elif scale == "large":
			return self.image_large_jump_distance
		else:
			return { "middle" : self.image_middle_jump_distance, "large" : self.image_large_jump_distance }


	#以下、ショートカットキー処理の関数

	#（タブ移動関数)leftが押された場合にタブを移動する関数
	def hotkey_left( self, event ):
		self.notebook.tab_jump( "left", 1 )

	#（タブ移動関数)shift+leftが押された場合にタブを移動する関数
	def hotkey_shift_left( self, event ):
		self.notebook.tab_jump( "left", self.tab_middle_jump_distance )

	#（タブ移動関数)alt+leftが押された場合にタブを移動する関数
	def hotkey_alt_left( self, event ):
		self.notebook.tab_jump( "left", self.tab_large_jump_distance )

	#（タブ移動関数)rightが押された場合にタブを移動する関数
	def hotkey_right( self, event ):
		self.notebook.tab_jump( "right", 1 )

	#（タブ移動関数)shift+rightが押された場合にタブを移動する関数
	def hotkey_shift_right( self, event ):
		self.notebook.tab_jump( "right", self.tab_middle_jump_distance )

	#（タブ移動関数)alt+rightが押された場合にタブを移動する関数
	def hotkey_alt_right( self, event ):
		self.notebook.tab_jump( "right", self.tab_large_jump_distance )

	#（画像の移動関数)upが押された場合に画像を移動する関数
	def hotkey_up( self, event ):
		#このGetCurrentPage()を使うことで、「タブが現在表示している」パネルの関数が使える
		self.notebook.GetCurrentPage().image_jump( "previous", 1 )

	#（画像の移動関数)pageupが押された場合に画像を移動する関数
	def hotkey_pageup( self, event ):
		self.notebook.GetCurrentPage().image_jump( "previous", 1 )

	#（画像の移動関数)shift+upが押された場合に画像を移動する関数
	def hotkey_shift_up( self, event ):
		self.notebook.GetCurrentPage().image_jump( "previous", self.image_middle_jump_distance )

	#（画像の移動関数)shift+pageupが押された場合に画像を移動する関数
	def hotkey_shift_pageup( self, event ):
		self.notebook.GetCurrentPage().image_jump( "previous", self.image_middle_jump_distance )

	#（画像の移動関数)alt+upが押された場合に画像を移動する関数
	def hotkey_alt_up( self, event ):
		self.notebook.GetCurrentPage().image_jump( "previous", self.image_large_jump_distance )

	#（画像の移動関数)alt+pageupが押された場合に画像を移動する関数
	def hotkey_alt_pageup( self, event ):
		self.notebook.GetCurrentPage().image_jump( "previous", self.image_large_jump_distance )

	#（画像の移動関数)downが押された場合に画像を移動する関数
	def hotkey_down( self, event ):
		self.notebook.GetCurrentPage().image_jump( "next", 1 )

	#（画像の移動関数)pagedownが押された場合に画像を移動する関数
	def hotkey_pagedown( self, event ):
		self.notebook.GetCurrentPage().image_jump( "next", 1 )

	#（画像の移動関数)shift+downが押された場合に画像を移動する関数
	def hotkey_shift_down( self, event ):
		self.notebook.GetCurrentPage().image_jump( "next", self.image_middle_jump_distance )

	#（画像の移動関数)shift+pagedownが押された場合に画像を移動する関数
	def hotkey_shift_pagedown( self, event ):
		self.notebook.GetCurrentPage().image_jump( "next", self.image_middle_jump_distance )

	#（画像の移動関数)alt+downが押された場合に画像を移動する関数
	def hotkey_alt_down( self, event ):
		self.notebook.GetCurrentPage().image_jump( "next", self.image_large_jump_distance )

	#（画像の移動関数)alt+pagedownが押された場合に画像を移動する関数
	def hotkey_alt_pagedown( self, event ):
		self.notebook.GetCurrentPage().image_jump( "next", self.image_large_jump_distance )

	#（画像の単独移動関数)ctrl+upが押された場合に画像の位置を「常に一つだけ」移動する
	def hotkey_ctrl_up( self, event ):
		self.notebook.GetCurrentPage().image_jump( "previous", 1, "no-adjust" )

	#（画像の単独移動関数)ctrl+downが押された場合に画像の位置を「常に一つだけ」移動する
	def hotkey_ctrl_down( self, event ):
		self.notebook.GetCurrentPage().image_jump( "next", 1, "no-adjust" )

	#（画像の単独移動関数)ctrl+pageupが押された場合に画像の位置を「常に一つだけ」移動する
	def hotkey_ctrl_pageup( self, event ):
		self.notebook.GetCurrentPage().image_jump( "previous", 1, "no-adjust" )

	#（画像の単独移動関数)ctrl+pagedownが押された場合に画像の位置を「常に一つだけ」移動する
	def hotkey_ctrl_pagedown( self, event ):
		self.notebook.GetCurrentPage().image_jump( "next", 1, "no-adjust" )

	#ctrl+wが押された場合に、現在表示中のタブを閉じる関数
	def hotkey_ctrl_w( self, event ):

		#タブがないときに閉じようとすると異常終了するので、タブ数が0を超える場合にのみ閉じる
		if self.notebook.GetPageCount() > 0:
			self.notebook.DeletePage( self.notebook.GetSelection() )

	#ctrl+dが押された場合に画像を並べる方向を反転する関数
	def hotkey_ctrl_d( self, event ):
		self.notebook.GetCurrentPage().toggle_page_direction()

	#alt+sが押された場合にステータスバー表示・非表示を切り替える関数
	def hotkey_alt_s( self, event ):
		self.statusbar_show_or_hide()

	#F5が押された場合に現在のパネルのファイルリストを再取得する
	def hotkey_f5( self, event ):
		self.notebook.GetCurrentPage().reload_filelist()

	#homeが押された場合に左端のタブに移動する
	def hotkey_home( self, event ):
		self.notebook.SetSelection(0)

	#endが押された場合に右端のタブに移動する
	def hotkey_end( self, event ):
		self.notebook.SetSelection( self.notebook.GetPageCount() - 1 )

	#ctrl + homeが押された場合に最初の画像に戻る
	def hotkey_ctrl_home( self, event ):
		self.notebook.GetCurrentPage().page_jump( 1 )

	#ctrl + endが押された場合に最後の画像に移る
	def hotkey_ctrl_end( self, event ):
		self.notebook.GetCurrentPage().page_jump( -1 )

	#f11が押された場合に全画面表示を切り替える関数
	def hotkey_f11( self, event ):
		if self.IsFullScreen() is True:
			self.ShowFullScreen( False )
		else:
			self.ShowFullScreen( True )


#拡張notebook（タブ部分）
#タブをいくつも飛ばすジャンプ、などを加えて拡張した
class extended_notebook( wx.Notebook ):

	#右クリックメニューのID
	notebook_rmenu_ID1 = wx.NewIdRef().GetId()
	notebook_rmenu_ID2 = wx.NewIdRef().GetId()

	def __init__( self, parent, id ):
		wx.Notebook.__init__( self, parent, id )

		self.parent_object = parent

		self.panel_count = 0

		#マウスホイール回転をパネルの中継関数に送る。
		self.Bind( wx.EVT_MOUSEWHEEL, self.wheel_to_panel )

		self.Bind( wx.EVT_NOTEBOOK_PAGE_CHANGED, self.panel_infomations_to_statusbar )

		#右クリックのバインド
		self.Bind( wx.EVT_RIGHT_UP, self.notebook_right_click_menu )


	#画像表示パネルを新規作成してタブに追加する関数
	def add_panel( self, picture_path, insert_position = -1 ):
		if insert_position < 0:
			insert_position = self.GetPageCount()

		if os.path.exists( picture_path ) and os.path.isfile( picture_path ):
			#タブのタイトル用に画像のフルパスからファイル名を取得する
			pic_filename = os.path.basename( picture_path )
			if reference_values.get_show_extension() is False:
				pic_filename = os.path.splitext( pic_filename )[0]

			#画像パネルを新しいタブとして追加
			##Ubuntu18.04(Python3+wxPython4)では、どうやら半角英数字だけのタイトルのタブと
			##全角混じりのタブ間＆画像間の移動で多少もたつくようだ。
			##いずれ修正されるかもしれないが、とりあえずの対策としてタブタイトルに・をつけておく。
			##（ここと画像移動の2箇所）
			self.InsertPage( insert_position, view_panel( self, wx.ID_ANY, picture_path, self.panel_count ), "・" + pic_filename )
			self.panel_count += 1

		else:
			reference_values.add_existence_error( picture_path )


	#どのタブを見るかを左右に移動する関数
	def tab_jump( self, direction, distance ):

		if distance == 1:
			tab_jump_distance = 1

		else:
			#self.GetSelection()：現在のタブの位置
			#self.GetPageCount()：全体のタブの数
			#剰余を使っているのは、例えば3つだけタブがあるのに50個のタブを移動する
			#といった、移動量が何周もする場合を想定したため。
			tab_jump_distance = distance % self.GetPageCount()

		#タブのアクティブ位置を左に移動
		if direction == "left":

			#タブ数の範囲内でのジャンプの場合
			if self.GetSelection() - tab_jump_distance >= 0:
				self.SetSelection( self.GetSelection() - tab_jump_distance )
				self.SetFocus()

			#左端を超えて一周回る場合
			else:
				self.SetSelection( self.GetPageCount() - ( tab_jump_distance - self.GetSelection() ) )
				self.SetFocus()

		#タブのアクティブ位置を右に移動
		elif direction == "right":

			#タブ数の範囲内でのジャンプの場合
			if self.GetSelection() + tab_jump_distance < self.GetPageCount():
				self.SetSelection( self.GetSelection() + tab_jump_distance )
				self.SetFocus()

			#右端を超えて一周回る場合
			else:
				self.SetSelection( self.GetSelection() + tab_jump_distance - self.GetPageCount() )
				self.SetFocus()


	#タブが受けたマウスイベントをパネルに転送する。
	def wheel_to_panel( self, event ):
		GetCurrentPage().wheel_to_pagemove( event )

	#パネルのステータスバー更新関数を起動するだけの転送関数
	def panel_infomations_to_statusbar( self, event ):
		self.GetCurrentPage().panel_infomations_to_statusbar()

	#全体のタブ数と現在のタブの位置を返す
	def tab_position(self):
		return str( self.GetSelection() + 1 ) + " ／ " + str( self.GetPageCount() )

	#パネルがリサイズされた時に、呼び出し元のパネルが現在表示中のタブだった場合のみステータスバー更新をさせる関数。
	#それぞれのパネル内で直接更新させると、リサイズのたびに全タブのパネルが
	#一斉にステータスバーへ情報を書き込むので、表示中以外のタブのデータで上書きされてしまう。
	def current_panel_to_statusbar( self, panel_number ):
		if panel_number == self.GetCurrentPage().get_own_panel_number():
			self.GetCurrentPage().panel_infomations_to_statusbar()

	#すべてのタブで画像ファイルのリストを更新する（パネルの右クリックから呼び出す用）
	def alltab_reload_list( self ):
		for i in range( self.GetPageCount() ):
			self.GetPage( i ).reload_filelist( i )


	#以下、右クリックメニュー関連
	def notebook_right_click_menu( self, event ):
		menu = wx.Menu()

		menu.Append( self.notebook_rmenu_ID1, "ファイルを開く（画像ファイル or フルパスのリストファイル）" )
		menu.Append( self.notebook_rmenu_ID2, "全画面表示を切り替え" )

		self.Bind( wx.EVT_MENU, self.notebook_rc_fileopen, id=self.notebook_rmenu_ID1 )
		self.Bind( wx.EVT_MENU, self.notebook_rc_toggle_fullscreen, id=self.notebook_rmenu_ID2 )

		self.PopupMenu( menu )
		menu.Destroy()


	#右クリックからファイルを開く
	def notebook_rc_fileopen( self, event ):
		self.parent_object.fileopen_dialog()

	#全画面表示切り替え
	def notebook_rc_toggle_fullscreen( self, event ):
		if self.parent_object.IsFullScreen() is True:
			self.parent_object.ShowFullScreen( False )
		else:
			self.parent_object.ShowFullScreen( True )
		


#画像を表示できるように拡張したパネル
class view_panel( wx.Panel ):

#＜基本構造＞
#それぞれのパネルが表示の基本となる画像データ「self.pil_image」を保持する。
#「self.pil_image」は、与えられたパスの画像と、そのディレクトリ下の次の画像を元にして
#外部の関数「picture_narrow2_or_broad1」を用いて連結したものである（横長画像であれば2つめの画像は使わない。単体の画像になる）。
#その基本画像データを、ウィンドウの大きさに合わせて拡大縮小して表示する。
#ファイル位置を送るか戻すかするなら、picture_narrow2_or_broad1関数に新しい画像のパスを渡して「self.pil_image」を更新する。

	#右クリックメニューのID
	rmenu_ID1 = wx.NewIdRef().GetId()
	rmenu_ID2 = wx.NewIdRef().GetId()
	rmenu_ID3 = wx.NewIdRef().GetId()
	rmenu_ID4 = wx.NewIdRef().GetId()
	rmenu_ID5 = wx.NewIdRef().GetId()
	rmenu_ID6 = wx.NewIdRef().GetId()
	rmenu_ID7 = wx.NewIdRef().GetId()
	rmenu_ID8 = wx.NewIdRef().GetId()
	rmenu_ID9 = wx.NewIdRef().GetId()
	rmenu_ID10 = wx.NewIdRef().GetId()
	rmenu_ID11 = wx.NewIdRef().GetId()
	rmenu_ID12 = wx.NewIdRef().GetId()
	rmenu_ID13 = wx.NewIdRef().GetId()
	rmenu_ID14 = wx.NewIdRef().GetId()
	rmenu_ID15 = wx.NewIdRef().GetId()


	def __init__( self, parent, id, picture_path, panel_number ):
		wx.Panel.__init__(self, parent, id)

		self.parent_object = parent

		self.pil_image = Image
		self.picture_file_list = []
		self.current_picture_file = os.path.basename( picture_path )
		self.directory = os.path.dirname( picture_path )
		self.display_resize_rate = 1
		self.borderline_rate = reference_values.get_borderline_rate()
		self.page_direction = reference_values.get_page_direction()
		self.unique_panel_number = panel_number #パネルリサイズ時にすべてのパネルがステータスバーにデータを書き込んでしまうため、これで現在表示中のタブのパネルなのか識別する。
		self.displayed_picture_name = [] #この行が無くても支障はない
		self.wx_bitmap_image = wx.Bitmap( 100, 100 ) #この行が無くても支障はない
		self.display_position = [ 0, 0 ] #この行が無くても支障はない
		self.picture_infomations = {}
		self.effects = copy.deepcopy( reference_values.get_base_effects() )

		#画像ファイルのリスト取得
		for i in maybe_filelist_natsort( os.listdir( self.directory ) ):
			filepath = os.path.join( self.directory, i )

			if os.path.isfile( filepath ) :
				#画像のmimetypeになっているファイルのみをまとめる
				if re.match( "image/", str( mimetypes.guess_type( filepath )[0] ) ):
					self.picture_file_list.append(i)

		#移動量0の進み戻り関数により、画像を読み込んで並べる（つまりself.pil_imageの取得）。
		#引数no-refreshをつけて画像表示とタブ名は更新しない。
		#どうせ直後にリサイズになり再表示するので、重複処理で遅延させたくないのが理由。
		self.page_move( 0, "no-refresh" )

		#起動直後に画像がいつまで待っても表示されないようなら、次行のコメント化を解除する必要があるかもしれない。
		#self.display_picture( None )

		#ウィンドウのリサイズ時にこのパネルに画像をリサイズして表示するように、イベントを設定する。
		self.Bind( wx.EVT_SIZE, self.display_picture )

		self.Bind( wx.EVT_MOUSEWHEEL, self.wheel_to_pagemove )

		self.Bind( wx.EVT_PAINT, self.OnPaint )

		#右クリックのバインド
		self.Bind( wx.EVT_RIGHT_UP, self.right_click_menu )


	#デバイスコンテキストを取得してパネルに画像を表示する
	def OnPaint(self, event=None):
		deviceContext = wx.PaintDC(self)
		deviceContext.Clear()
		deviceContext.SetPen( wx.Pen( wx.BLACK, 4 ) )
		deviceContext.DrawBitmap( self.wx_bitmap_image, 0, 0 )

	#このpanelに画像を表示する関数
	def display_picture( self, event ):

		panel_size_x = self.GetSize()[0]
		panel_size_y = self.GetSize()[1]

		#画像イメージの縦か横のどちらかがパネルより大きい、つまり縮小のみPILで行う
		if self.pil_image.size[0] > panel_size_x or self.pil_image.size[1] > panel_size_y:
			#FlatNotebookにすると0による除算になってしまうので、その対策
			if panel_size_x == 0 or panel_size_y == 0:
				adjusted_pil_image = self.pil_image
				self.display_resize_rate = 1
			else:
				#画像とウィンドウの縦横比を比較し、ウィンドウの縦と横の長さのどちらに合わせて画像をリサイズするかを分ける
				if 1.0 * self.pil_image.size[0] / self.pil_image.size[1] > 1.0 * panel_size_x / panel_size_y:
					adjusted_pil_image = self.pil_image.resize( ( panel_size_x, int( self.pil_image.size[1] * panel_size_x / self.pil_image.size[0] ) ), Image.LANCZOS )
					self.display_resize_rate = 1.0 * panel_size_x / self.pil_image.size[0]
				else:
					adjusted_pil_image = self.pil_image.resize( ( int( self.pil_image.size[0] * panel_size_y / self.pil_image.size[1] ), panel_size_y ), Image.LANCZOS )
					self.display_resize_rate = 1.0 * panel_size_y / self.pil_image.size[1]

		else:
			adjusted_pil_image = self.pil_image
			self.display_resize_rate = 1

		#PILイメージをwx.Imageに変換する
		wx_image = wx.Image( adjusted_pil_image.size[0], adjusted_pil_image.size[1] )
		wx_image.SetData( adjusted_pil_image.convert('RGB').tobytes() )

		wx_image_size_x = wx_image.GetSize()[0]
		wx_image_size_y = wx_image.GetSize()[1]

		#画像が（縦横の両方とも）パネルより小さい、つまり拡大のみwxで行う
		if wx_image_size_x < panel_size_x and wx_image_size_y < panel_size_y:
			#画像とウィンドウの縦横比を比較し、ウィンドウの縦と横の長さのどちらに合わせて画像をリサイズするかを分ける
			if 1.0 * wx_image_size_x / wx_image_size_y > 1.0 * self.GetSize()[0] / self.GetSize()[1]:
				rescale_size_x = panel_size_x
				rescale_size_y = int( wx_image_size_y * panel_size_x / wx_image_size_x )
				self.display_resize_rate = 1.0 * panel_size_x / wx_image_size_x
			else:
				rescale_size_x = int( wx_image_size_x * panel_size_y / wx_image_size_y )
				rescale_size_y = panel_size_y
				self.display_resize_rate = 1.0 * panel_size_y / wx_image_size_y
			wx_image = wx_image.Rescale(rescale_size_x,rescale_size_y,quality=wx.IMAGE_QUALITY_HIGH)

		wx_image_size_x = wx_image.GetSize()[0]
		wx_image_size_y = wx_image.GetSize()[1]
		#FlatNotebookにすると0による除算になってしまうので、その対策
		if wx_image_size_y == 0 or panel_size_x == 0:
			display_pos_x = 0
			display_pos_y = 0
		else:
			#画像とウィンドウの縦横比を比較し、中央に来るように表示位置を求める
			if 1.0 * wx_image.GetSize()[0] / wx_image.GetSize()[1] > 1.0 * panel_size_x / panel_size_y:
				display_pos_x = 0
				#（余白の長さ／2）で中央に表示するための（上端からの）表示ピクセル数を求める。
				display_pos_y = int( ( panel_size_y - wx_image.GetSize()[1] ) / 2 )
			else:
				display_pos_x = int( ( panel_size_x - wx_image.GetSize()[0] ) / 2 )
				display_pos_y = 0

		wx_resized_plus_background = reference_values.get_background_image().Copy()
		#背景に画像を貼り付けて表示用イメージを作成する。
		wx_resized_plus_background.Paste( wx_image, display_pos_x, display_pos_y )

		#wx.StaticBitmapで表示させると、画像を何十枚か送るにつれて表示されるまでの
		#タイムラグが大きくなっていくので、wx.PaintDCとやらを使って表示させることにした。
		self.wx_bitmap_image =  wx_resized_plus_background.ConvertToBitmap()
		self.Refresh()

		#notebookに判別してもらい、現在表示中のパネルの場合のみステータスバーを更新する。
		self.parent_object.current_panel_to_statusbar( self.unique_panel_number )


	#ホイール回転をpage_moveに送る中継関数
	def wheel_to_pagemove( self, event ):

		if int( event.GetModifiers() ) == 0:
			file_position = self.picture_file_list.index( self.current_picture_file )

			if event.GetWheelRotation() > 0:
				#2つ前と1つ前の画像の位置を取得
				if file_position == 0:
					before_position_pair = [ len( self.picture_file_list ) - 2, len( self.picture_file_list ) - 1 ]
				elif file_position == 1:
					before_position_pair = [ len( self.picture_file_list ) - 1, 0]
				else:
					before_position_pair = [ file_position - 2, file_position - 1 ]

				#1つと2つ前の画像が共に縦長で、2枚同時に表示されるようなら（並べる関数から返ってきた画像情報に2枚目の情報があるなら）2枚分戻る。
				if "name2" in picture_narrow2_or_broad1( os.path.join( self.directory , self.picture_file_list[ before_position_pair[0] ] ), os.path.join( self.directory , self.picture_file_list[ before_position_pair[1] ] ), self.page_direction, self.borderline_rate, self.effects, reference_values.picture_bgcolor, testmode = True )[1]:
					self.page_move( -2 )
				else:
					self.page_move( -1 )
			else:
				#画像を1枚表示しているか2枚なのかで、送る枚数を変える。（画像情報に2枚目の情報があるかどうかで判断する）
				#（修飾キーなしのキー操作のみ。修飾キーありやマウスホイール回転は無関係に1枚ずつとする）
				if "name2" in self.picture_infomations:
					self.page_move( 2 )
				else:
					self.page_move( 1 )

		#Shiftが押されていれば、中規模移動
		elif event.GetModifiers() == wx.MOD_SHIFT:
			if event.GetWheelRotation() > 0:
				self.page_move( -1 * self.parent_object.parent_object.get_image_jump_distance( "middle" ) )
			else:
				self.page_move( self.parent_object.parent_object.get_image_jump_distance( "middle" ) )

		#Altが押されていれば、大規模移動
		elif event.GetModifiers() == wx.MOD_ALT:
			if event.GetWheelRotation() > 0:
				self.page_move( -1 * self.parent_object.parent_object.get_image_jump_distance( "large" ) )
			else:
				self.page_move( self.parent_object.parent_object.get_image_jump_distance( "large" ) )

		#Ctrlが押されていれば、常に1枚ずつ移動
		elif event.GetModifiers() == wx.MOD_CONTROL:
			if event.GetWheelRotation() > 0:
				self.page_move( -1 )
			else:
				self.page_move( 1 )


	#page_move関数を使って、どの画像を表示するかを移動（ショートカットキーからの操作用）
	def image_jump( self, direction, distance, adjust_with_apposing = "" ):

		#表示中にディレクトリや全画像を消してしまうと移動しようがないので、事前に確認する
		if not self.current_picture_file in self.picture_file_list:

			if not os.path.isdir( self.directory ):
				error_dialog =  wx.MessageDialog( None, "ディレクトリが存在していません\n\n表示中の画像があったディレクトリ ： " + self.directory, "Error",wx.OK | wx.ICON_ERROR )
				error_dialog.ShowModal()
				error_dialog.Destroy()
				return False

			else:
				self.reload_filelist()

				if self.picture_file_list == []:
					return False

		#移動量が1ではない（中・大ジャンプ）、ないし、移動量を
		#調整しない指定（強制的に1枚だけの移動）なら、単純に移動する
		if distance != 1 or adjust_with_apposing == "no-adjust":

			if direction == "previous":
				self.page_move( distance * -1 )

			elif direction == "next":
				self.page_move( distance )

		#移動量が1だけで、調整が否定されない（adjust_with_apposingがno-adjustでない）
		#場合のみ、画像が並ぶか単体表示かで移動量を調整する
		else:

			file_position = self.picture_file_list.index( self.current_picture_file )
		
			if direction == "previous":

				#2つ前と1つ前の画像の位置を取得
				if file_position == 0:
					before_position_pair = [ len( self.picture_file_list ) - 2, len( self.picture_file_list ) - 1 ]
				elif file_position == 1:
					before_position_pair = [ len( self.picture_file_list ) - 1, 0]
				else:
					before_position_pair = [ file_position - 2, file_position - 1 ]

				#1つと2つ前の画像が共に縦長で、2枚同時に表示されるようなら
				#（並べる関数が返した画像情報に2枚目の情報があるなら）2枚分戻る。
				if "name2" in picture_narrow2_or_broad1( os.path.join( self.directory , self.picture_file_list[ before_position_pair[0] ] ), os.path.join( self.directory , self.picture_file_list[ before_position_pair[1] ] ), self.page_direction, self.borderline_rate, self.effects, reference_values.picture_bgcolor, testmode = True )[1]:
					self.page_move( -2 )
				else:
					self.page_move( -1 )

			elif direction == "next":

				#画像を1枚表示しているか2枚なのかで、送る枚数を変える。
				#（現在の画像情報に2枚目の情報があるかどうかで判断する）
				if "name2" in self.picture_infomations:
					self.page_move( 2 )
				else:
					self.page_move( 1 )


	#ページを進めたり戻ったりする関数
	def page_move( self, next_or_prev, refresh_yesno = "refresh" ):

		if next_or_prev > 0:
		#後の画像ファイルへと現在位置を進める場合

			#剰余を使っているのは、例えば3つだけ画像ファイルがあるのに10個分移動する、といった移動量が何周もする場合を想定したため。
			#なのだが、負の数の剰余が予想どおりではなかったので、少々厄介なことになった（後述）。
			next_or_prev = next_or_prev % len( self.picture_file_list )

			if self.picture_file_list.index( self.current_picture_file ) + next_or_prev <= len( self.picture_file_list ) - 1:
				self.current_picture_file = self.picture_file_list[ self.picture_file_list.index( self.current_picture_file ) + next_or_prev ]
			else:
			#進んだ結果、ファイルリストの最後を超えてしまう場合。先頭からに戻る。
				self.current_picture_file = self.picture_file_list[ self.picture_file_list.index( self.current_picture_file ) + next_or_prev - len( self.picture_file_list ) ]

			#進んだ先のファイルが存在しなかった場合。
			#存在するファイルに当たるまで1つずつ進みながら、読めないファイルをリストから削除する。
			if not os.path.exists( os.path.join( self.directory ,self.current_picture_file ) ):
				while not os.path.exists( os.path.join( self.directory ,self.current_picture_file ) ) and self.picture_file_list != []:
					delete_file = self.current_picture_file

					if self.picture_file_list.index( self.current_picture_file ) + 1 > len( self.picture_file_list ) - 1:
						self.current_picture_file = self.picture_file_list[ 0 ]
					else:
						self.current_picture_file = self.picture_file_list[ self.picture_file_list.index( self.current_picture_file ) + 1 ]

					self.picture_file_list.remove( delete_file )

				if self.picture_file_list == []:
					error_dialog =  wx.MessageDialog( None, "ディレクトリ「" + self.directory + "」に画像が存在していません", "Error",wx.OK | wx.ICON_ERROR )
					error_dialog.ShowModal()
					error_dialog.Destroy()
					return False

		elif next_or_prev < 0:
		#前の画像ファイルへと戻る場合

			#負の数の剰余が予想どおりではなかったので、-1をかけて正の数にしてから
			#剰余を計算して、また-1をかけ直して負の数に戻すというややこしいことをしている。
			#正の数の剰余の正負反転だろうと思っていたが、正の分母+負の分子になっていた。
			next_or_prev = ( ( -1 * next_or_prev ) % len( self.picture_file_list ) ) * -1

			if self.picture_file_list.index( self.current_picture_file ) + next_or_prev % len( self.picture_file_list ) < 0:
			#戻ったら先頭（ファイルリストの位置が0）よりも戻りすぎる場合。
				self.current_picture_file = self.picture_file_list[ len( self.picture_file_list ) + next_or_prev ]
			else:
				self.current_picture_file = self.picture_file_list[ self.picture_file_list.index( self.current_picture_file ) + next_or_prev ]

			#戻った先のファイルが存在しなかった場合。
			#存在するファイルに当たるまで1つずつ戻りながら、読めないファイルをリストから削除する。
			if not os.path.exists( os.path.join( self.directory ,self.current_picture_file ) ):
				while not os.path.exists( os.path.join( self.directory ,self.current_picture_file ) ) and self.picture_file_list != []:
					delete_file = self.current_picture_file

					if self.picture_file_list.index( self.current_picture_file ) - 1 < 0:
						self.current_picture_file = self.picture_file_list[ len( self.picture_file_list ) - 1 ]
					else:
						self.current_picture_file = self.picture_file_list[ self.picture_file_list.index( self.current_picture_file ) - 1 ]

					self.picture_file_list.remove( delete_file )

				if self.picture_file_list == []:
					error_dialog =  wx.MessageDialog( None, "ディレクトリ「" + self.directory + "」に画像が存在していません", "Error",wx.OK | wx.ICON_ERROR )
					error_dialog.ShowModal()
					error_dialog.Destroy()
					return False

		else:
			#next_or_prev == 0 の場合（ファイル位置を移動しない。つまり起動直後の最初の画像表示など）
			pass


		#次にある画像のファイル名を取得。引数の画像が最後なら、次の画像は最初のファイルとする。
		if self.picture_file_list.index( self.current_picture_file ) != len( self.picture_file_list ) - 1:
			second_picture = self.picture_file_list[ self.picture_file_list.index( self.current_picture_file ) + 1 ]
		else:
			second_picture = self.picture_file_list[ 0 ]

		#画像のフルパスを構築。ディレクトリ内に縦長画像が1つだけだと、同じ画像が2つ並んでしまうので、2つめのパスをNoneにする。
		image_path1 = os.path.join( self.directory , self.current_picture_file )
		if len( self.picture_file_list ) == 1:
			image_path2 = None
		else:
			image_path2 = os.path.join( self.directory , second_picture )

		#与えられた画像と（縦横比次第で）次の画像を並べたPILイメージを取得する
		temporary_data = picture_narrow2_or_broad1( image_path1, image_path2, self.page_direction, self.borderline_rate, self.effects, reference_values.get_bgcolor() )
		self.pil_image = temporary_data[0]
		self.picture_infomations = temporary_data[1]

		if refresh_yesno != "no-refresh":
			#パネルへの画像の再表示
			#〜( self, event ):になっている関数でも、eventの代わりにNoneでも渡せば
			#eventとは無関係に単独で関数を使用できるようだ
			self.display_picture( None )

			#タブに表示されているファイル名を更新する
			##Ubuntu18.04(Python3+wxPython4)では、どうやら半角英数字だけのタイトルのタブと
			##全角混じりのタブ間＆画像間の移動で多少もたつくようだ。
			##いずれ修正されるかもしれないが、とりあえずの対策としてタブタイトルに・をつけておく。
			##（こことタブ移動の2箇所）
			if reference_values.get_show_extension() is True:
				self.parent_object.SetPageText( self.parent_object.GetSelection(), "・" + self.current_picture_file )
			else:
				self.parent_object.SetPageText( self.parent_object.GetSelection(), "・" + os.path.splitext( self.current_picture_file )[0] )


	#page_move関数のような進む戻るではなく、指定した位置の画像に直接移動する関数
	#画像位置は0ではなく1から始まるので注意（最初の画像はposition=1）
	#position=-1など、画像リストの範囲外の数値なら、最後の画像に移動する
	def page_jump( self, position ):

		if position > 0 and position <= len( self.picture_file_list ):
			self.current_picture_file = self.picture_file_list[ position - 1 ]
		else:
			self.current_picture_file = self.picture_file_list[ len( self.picture_file_list ) - 1 ]

		self.page_move( 0 )


	#画像を並べる方向を逆転
	def toggle_page_direction( self ):
		if self.page_direction == "LeftToRight":
			self.page_direction = "RightToLeft"
		else:
			self.page_direction = "LeftToRight"

		#以前はここで画像を読み込み2つ並べたイメージ(self.pil_image)を取得していたが
		#画像の進み戻り関数と処理が重なるので一本化した。
		self.page_move( 0 )


	#ファイル位置、タブ位置、拡大縮小率などをステータスバーに表示する
	def panel_infomations_to_statusbar( self ):

		#画像位置表記の構築
		if "name2" in self.picture_infomations:
		#画像を2つ並べて表示している場合
			if self.page_direction == "LeftToRight":
				#左から右なので、1つめの画像の位置情報が先に来る
				file_position = str( self.picture_file_list.index( self.current_picture_file ) + 1 ) + " ＆ " + str( self.picture_file_list.index( self.picture_infomations.get( "name2" ) ) + 1 )
			else:
				file_position = str( self.picture_file_list.index( self.picture_infomations.get( "name2" ) ) + 1 ) + " ＆ " + str( self.picture_file_list.index( self.current_picture_file ) + 1 )
		else:
		#表示されている画像が1つだけの場合の拡大縮小率
			file_position = str( self.picture_file_list.index( self.current_picture_file ) + 1 )

		file_position_and_totalnumber = file_position + " ／ " + str( len( self.picture_file_list ) )

		#拡大縮小率表記の構築
		if "resize-rate2" in self.picture_infomations:
		#画像を2つ並べて表示している場合
			if self.page_direction == "LeftToRight":
				#左から右なので、1つめの画像の拡大縮小率が先に来る
				resize_rate_percentage = str( int( self.picture_infomations.get( 'resize-rate1' ) * self.display_resize_rate * 100 ) ) + "％" + " 、 " + str( int( self.picture_infomations.get( 'resize-rate2' ) * self.display_resize_rate * 100 ) ) + "％"
			else:
				resize_rate_percentage = str( int( self.picture_infomations.get( 'resize-rate2' ) * self.display_resize_rate * 100 ) ) + "％" + " 、 " + str( int( self.picture_infomations.get( 'resize-rate1' ) * self.display_resize_rate * 100 ) ) + "％"
		else:
		#表示されている画像が1つだけの場合の拡大縮小率
			resize_rate_percentage = str( int( self.picture_infomations.get( 'resize-rate1' ) * self.display_resize_rate * 100 ) ) + "％"

		#画像を並べる方向の表示部分の作成
		if self.page_direction == "LeftToRight":
			page_direction_status = "左から右"
		else:
			page_direction_status = "右から左"

		#ファイル名表示の構築
		if "name2" in self.picture_infomations:
			if self.page_direction == "LeftToRight":
				#左から右なので、1つめのファイル名が先に来る
				filename = "（左）" + self.picture_infomations.get( 'name1' ) + " 、 " + "（右）" +  self.picture_infomations.get( 'name2' )
			else:
				filename = "（左）" + self.picture_infomations.get( 'name2' ) + " 、 " + "（右）" +  self.picture_infomations.get( 'name1' )
		elif "name1" in self.picture_infomations:
			filename = self.picture_infomations.get( 'name1' )
		else:
			filename = "ファイル名取得エラー："

		#効果の内容をまとめる
		effects = ""
		if len( self.effects ) != 0:
			effects += " 、 効果 ［"
			if "rotate" in self.effects:
				effects += " 回転：" + str( self.effects.get( "rotate" ) ) + "°"
			if "horizontal-reverse" in self.effects:
				effects += " 左右反転"
			if "vertical-reverse" in self.effects:
				effects += " 上下反転"
			if "brightness" in self.effects:
				effects += " 明るさ：" + str( self.effects.get( "brightness" ) ) + "％"
			if "contrast" in self.effects:
				effects += " コントラスト：" + str( self.effects.get( "contrast" ) ) + "％"
			effects += " ］"

		statusbar_data = "ファイル位置：" + file_position_and_totalnumber + " 、 " + "タブ位置：" + self.parent_object.tab_position() + " 、 " + "拡大縮小率：" + resize_rate_percentage + " 、 " + "画像を並べる方向：" + page_direction_status+ " 、 " + "ファイル名：" + filename + effects
		self.parent_object.parent_object.statusbar.SetStatusText( statusbar_data )


	#このパネルの識別番号を返すだけ
	def get_own_panel_number( self ):
		return self.unique_panel_number


	#以下、右クリックメニュー関連
	def right_click_menu( self, event ):
		menu = wx.Menu()

		menu.Append( self.rmenu_ID1, "ファイルを開く（画像ファイル or フルパスのリストファイル）" )
		menu.Append( self.rmenu_ID2, "現タブの右に開く（リストで大量に開くと順番が飛ぶかも）" )
		menu.Append( self.rmenu_ID3, "全画面表示を切り替え" )
		menu.Append( self.rmenu_ID4, "ステータスバーの表示切り替え （Alt + S）" )
		menu.Append( self.rmenu_ID5, "画像を並べる方向を反転する（現在のタブのみ） （Ctrl + D）" )
		menu.Append( self.rmenu_ID6, "画像ファイルのリストを更新する（現在のタブのみ） （F5）" )
		menu.Append( self.rmenu_ID7, "すべてのタブでファイルリストを更新する" )
		menu.Append( self.rmenu_ID8, "指定したタブに移動" )
		menu.Append( self.rmenu_ID9, "指定した位置の画像に移動" )
		menu.Append( self.rmenu_ID10, "現在のタブを閉じる" )
		menu.Append( self.rmenu_ID11, "エラーになったファイルのリストを表示する" )
		menu.Append( self.rmenu_ID12, "ショートカットキーの一覧" )
		menu.Append( self.rmenu_ID13, "このタブの画像にかけるエフェクト等を変更する" )
		menu.Append( self.rmenu_ID14, "起動時の設定（設定ファイル）を編集する" )
		menu.Append( self.rmenu_ID15, "画像の情報を表示する" )
		#↑表示順を入れ替えているので、番号順に並んでいないことに注意

		self.Bind( wx.EVT_MENU, self.rc_fileopen, id=self.rmenu_ID1 )
		self.Bind( wx.EVT_MENU, self.rc_fileopen_rightside, id=self.rmenu_ID2 )
		self.Bind( wx.EVT_MENU, self.rc_toggle_fullscreen, id=self.rmenu_ID3 )
		self.Bind( wx.EVT_MENU, self.rc_toggle_statusbar, id=self.rmenu_ID4 )
		self.Bind( wx.EVT_MENU, self.rc_return_image_direction, id=self.rmenu_ID5 )
		self.Bind( wx.EVT_MENU, self.rc_reload_picturefile_list, id=self.rmenu_ID6 )
		self.Bind( wx.EVT_MENU, self.rc_alltab_reload_filelist, id=self.rmenu_ID7 )
		self.Bind( wx.EVT_MENU, self.rc_move_to_choiced_tab, id=self.rmenu_ID8 )
		self.Bind( wx.EVT_MENU, self.rc_move_to_choiced_imagelist_position, id=self.rmenu_ID9 )
		self.Bind( wx.EVT_MENU, self.rc_close_current_tab, id=self.rmenu_ID10 )
		self.Bind( wx.EVT_MENU, self.rc_show_errorfile_list, id=self.rmenu_ID11 )
		self.Bind( wx.EVT_MENU, self.rc_shortcutkey_list, id=self.rmenu_ID12 )
		self.Bind( wx.EVT_MENU, self.rc_effects, id=self.rmenu_ID13 )
		self.Bind( wx.EVT_MENU, self.rc_ini_setting, id=self.rmenu_ID14 )
		self.Bind( wx.EVT_MENU, self.rc_image_infomation, id=self.rmenu_ID15 )

		self.PopupMenu( menu )
		menu.Destroy()

	#右クリックから画像の情報を表示する
	def rc_image_infomation( self, event ):
		#効果の内容をまとめる
		effects = ""
		if len( self.effects ) != 0:
			effects += "\n\n［ このタブにかかっているエフェクト ］"
			if "rotate" in self.effects:
				effects += "\n　　回転：" + str( self.effects.get( "rotate" ) ) + "°"
			if "horizontal-reverse" in self.effects:
				effects += "\n　　左右反転"
			if "vertical-reverse" in self.effects:
				effects += "\n　　上下反転"
			if "brightness" in self.effects:
				effects += "\n　　明るさ：" + str( self.effects.get( "brightness" ) ) + "％"
			if "contrast" in self.effects:
				effects += "\n　　コントラスト：" + str( self.effects.get( "contrast" ) ) + "％"

		if "name2" in self.picture_infomations:
			info_text = "● 一枚目 （" + self.picture_infomations.get( 'direction1' ) +  "）\n\nファイル名 ： " + self.picture_infomations.get( 'name1' ) + "\n\nパス：" + self.picture_infomations.get( 'path1' ) + "\n\n画像サイズ（幅 x 高さ）：" + str( self.picture_infomations.get( 'width1' ) ) + " x " + str( self.picture_infomations.get( 'height1' ) ) + "\n表示する際の拡大縮小率：" + str( int( self.picture_infomations.get( 'resize-rate1' ) * self.display_resize_rate * 100 ) ) + "％" + "\n\nファイルサイズ：" + get_filesize( self.picture_infomations.get( 'path1' ) ) + "\n\n────────────────────────────────────────\n" + "● 二枚目 （" + self.picture_infomations.get( 'direction2' ) +  "）\n\nファイル名 ： " + self.picture_infomations.get( 'name2' ) + "\n\nパス：" + self.picture_infomations.get( 'path2' ) + "\n\n画像サイズ（幅 x 高さ）：" + str( self.picture_infomations.get( 'width2' ) ) + " x " + str( self.picture_infomations.get( 'height2' ) ) + "\n表示する際の拡大縮小率：" + str( int( self.picture_infomations.get( 'resize-rate2' ) * self.display_resize_rate * 100 ) ) + "％" + "\n\nファイルサイズ：" + get_filesize( self.picture_infomations.get( 'path2' ) ) + effects

		else:
			info_text = "ファイル名 ： " + self.picture_infomations.get( 'name1' ) + "\n\nパス：" + self.picture_infomations.get( 'path1' ) + "\n\n画像サイズ（幅 x 高さ）：" + str( self.picture_infomations.get( 'width1' ) ) + " x " + str( self.picture_infomations.get( 'height1' ) ) + "\n表示する際の拡大縮小率：" + str( int( self.picture_infomations.get( 'resize-rate1' ) * self.display_resize_rate * 100 ) ) + "％" + "\n\nファイルサイズ：" + get_filesize( self.picture_infomations.get( 'path1' ) ) + effects

		pictinfo_dialog = TextDialog( "画像の情報", ( 900, 740 ), info_text )
		#↓以下でも良い
		#import wx.lib.dialogs
		#pictinfo_dialog = wx.lib.dialogs.ScrolledMessageDialog( self, info_text, "画像の情報", wx.DefaultPosition, ( 800, 500 ) )

		pictinfo_dialog.ShowModal()
		pictinfo_dialog.Destroy()


	#右クリックからファイルを開く
	def rc_fileopen( self, event ):
		self.parent_object.parent_object.fileopen_dialog( self.directory )

	#右クリックから「現在のタブの右に」ファイルを開く
	def rc_fileopen_rightside( self, event ):
		self.parent_object.parent_object.fileopen_dialog( self.directory, self.parent_object.GetSelection() + 1 )

	#全画面表示切り替え
	def rc_toggle_fullscreen( self, event ):
		if self.parent_object.parent_object.IsFullScreen() is True:
			self.parent_object.parent_object.ShowFullScreen( False )
		else:
			self.parent_object.parent_object.ShowFullScreen( True )

	#ステータスバーの表示切り替え
	def rc_toggle_statusbar( self, event ):
		self.parent_object.parent_object.statusbar_show_or_hide()

	#画像を並べる方向を反転する
	def rc_return_image_direction( self, event ):
		self.toggle_page_direction()

	#指定したタブに移動
	def rc_move_to_choiced_tab( self, event ):
		#まず、タブのタイトル一覧を取得する
		tab_title_list = []
		for i in range( self.parent_object.GetPageCount() ):
			tab_title_list.append( self.parent_object.GetPageText( i ) )

		slider_dialog = SliderDialog( "どのタブに移動しますか？", ( 800, 260 ), 1, self.parent_object.GetPageCount(), self.parent_object.GetSelection() + 1, "左端が1で右端が" + str( self.parent_object.GetPageCount() ) + "、表示中のタブは" + str( self.parent_object.GetSelection() + 1 ) + "です", "タブのタイトル", tab_title_list )

		if slider_dialog.ShowModal() == wx.ID_OK:
			slider_value = slider_dialog.get_slider_value()
			self.parent_object.SetSelection( slider_value - 1 )

		slider_dialog.Destroy()
		self.parent_object.SetFocus()

	#画像ファイルのリストを更新する
	def rc_reload_picturefile_list( self, event ):
		self.reload_filelist()

	#すべてのタブで画像ファイルのリストを更新する
	def rc_alltab_reload_filelist( self, event ):
		self.parent_object.alltab_reload_list()

	#指定したファイル位置の画像に移動
	def rc_move_to_choiced_imagelist_position( self, event ):
		slider_dialog = SliderDialog( "どの位置の画像表示に移動しますか？", ( 800, 260 ), 1, len( self.picture_file_list ), self.picture_file_list.index( self.current_picture_file ) + 1, "表示中の画像は" + str( self.picture_file_list.index( self.current_picture_file ) + 1 ) + "番目です", "ファイル名", self.picture_file_list )

		if slider_dialog.ShowModal() == wx.ID_OK:
			self.page_jump( slider_dialog.get_slider_value() )

		slider_dialog.Destroy()

	#右クリックからエラーになったファイルのリストを表示する
	def rc_show_errorfile_list( self, event ):
		list_text = "※ 相対パスの場合、絶対パスに変換しているので、元の記述と一致しない場合があります\n\n●存在しない画像ファイル\n　（正しく認識できないコマンドライン引数も、ファイル指定と区別できないのでここに入ります）\n\n"

		if len( reference_values.get_existence_error() ) == 0:
			list_text += "■現在、このエラーに該当するファイルはありません。■"
			list_text += "\n"

		else:
			for i in reference_values.get_existence_error():
				list_text += i
				list_text += "\n"

		list_text += "\n────────────────────────────────────────\n"
		list_text += "●読み込みエラーになったファイル\n　（拡張子がjpgなのに内容がテキストファイルだった場合などがこのエラーになります）\n\n"

		if len( reference_values.get_read_error() ) == 0:
			list_text += "■現在、このエラーに該当するファイルはありません。■"
			list_text += "\n"

		else:
			for i in reference_values.get_read_error():
				list_text += i
				list_text += "\n"

		errorfilelist_dialog = TextDialog( "エラーになったファイルのリスト", ( 800, 500 ), list_text )

		errorfilelist_dialog.ShowModal()
		errorfilelist_dialog.Destroy()

	#ショートカットキーの一覧を表示する
	def rc_shortcutkey_list( self, event ):
		key_list = "← ： 左のタブに移動\n"
		key_list += "→ ： 右のタブに移動\n"
		key_list += "Shift + ← ： 左のタブに10個移動\n"
		key_list += "Shift + → ： 右のタブに10個移動\n"
		key_list += "Alt + ← ： 左のタブに30個移動\n"
		key_list += "Alt + → ： 右のタブに30個移動\n\n"

		key_list += "↑ あるいは PageUp あるいは マウスホイール上回転 ： 前の画像に移動（画像が2つ並ぶなら1枚飛ばし）\n"
		key_list += "↓ あるいは PageDown あるいは マウスホイール下回転 ： 次の画像に移動（画像が2つ並ぶなら1枚飛ばし）\n"
		key_list += "Shift + ↑ あるいは PageUp あるいは マウスホイール上回転 ： 10枚前の画像に移動\n"
		key_list += "Shift + ↓ あるいは PageDown マウスホイール下回転 ： 10枚次の画像に移動\n"
		key_list += "Alt + ↑ あるいは PageUp あるいは マウスホイール上回転 ： 30枚前の画像に移動\n"
		key_list += "Alt + ↓ あるいは PageDown あるいは マウスホイール下回転 ： 30枚次の画像に移動\n\n"

		key_list += "Ctrl + ↑ あるいは PageUp あるいは マウスホイール上回転 ： 常に1つだけ前の画像に移動\n"
		key_list += "Ctrl + ↓ あるいは PageDown あるいは マウスホイール下回転 ： 常に1つだけ次の画像に移動\n\n"

		key_list += "Ctrl + W ： 現在のタブを閉じる\n"
		key_list += "Ctrl + D ： ページを並べる方向を逆にする\n"
		key_list += "Alt + S ： ステータスバー表示・非表示の切り替え\n"
		key_list += "F5 ： 現在のタブが扱うディレクトリのファイルリストを更新する\n\n"

		key_list += "F11 ： 全画面表示を切り替える\n\n"

		key_list += "Alt + F4 ： ウィンドウを閉じて終了する\n（作者が与えた機能ではなく、ウィンドウに最初から備わっているものです）\n"

		shortcutkey_dialog = TextDialog( "ショートカットキーの一覧", ( 800, 560 ), key_list )

		shortcutkey_dialog.ShowModal()
		shortcutkey_dialog.Destroy()

	#右クリックから画像にかける効果を変更する
	def rc_effects( self, event ):
		other_dict = { "borderline-rate" : self.borderline_rate }

		set_effects_dialog = EffectsDialog( "このタブの画像にかけるエフェクト", ( 800, 700 ), self.effects, other_dict )

		if set_effects_dialog.ShowModal() == wx.ID_OK:
			self.effects = set_effects_dialog.get_effects_dictionary()
			self.borderline_rate = set_effects_dialog.get_other_dictionary().get( "borderline-rate" )
			self.page_move( 0 )

		set_effects_dialog.Destroy()

	#INIファイルを生成、設定する
	def rc_ini_setting( self, event ):
		settings_dict = {}
		effects_dict = {}

		#INIファイルの内容を読み込む
		if os.path.isfile( reference_values.get_ini_path() ):
			inisetting = configparser.ConfigParser()
			inisetting.read( reference_values.get_ini_path() )

			for settingkey in [ "window-mode", "window-size", "window-position", "statusbar", "page-direction", "borderline-rate", "background-color-rgb", "show-extension", "drop-mode" ]:
				if inisetting.has_option( "settings", settingkey ):

					if settingkey == "window-size":
						size_set = inisetting.get( "settings", settingkey ).split( "x" )
						settings_dict[ "window-size-x" ] = int( size_set[0] )
						settings_dict[ "window-size-y" ] = int( size_set[1] )

					elif settingkey == "window-position":
						position_set = inisetting.get( "settings", settingkey ).split( "," )
						settings_dict[ "window-position-x" ] = int( position_set[0] )
						settings_dict[ "window-position-y" ] = int( position_set[1] )

					elif settingkey == "background-color-rgb":
						if re.match( "^[0-9]{1,3}[,][0-9]{1,3}[,][0-9]{1,3}", inisetting.get( "settings", settingkey ) ):
							bg_color_set = inisetting.get( "settings", settingkey ).split( "," )
							settings_dict[ "bgcolor-r" ] = int( bg_color_set[0] )
							settings_dict[ "bgcolor-g" ] = int( bg_color_set[1] )
							settings_dict[ "bgcolor-b" ] = int( bg_color_set[2] )

					elif settingkey == "borderline-rate":
						settings_dict[ "borderline-rate" ] = float( inisetting.get( "settings", settingkey ) )

					elif settingkey == "statusbar" or settingkey == "show-extension":
						settings_dict[ settingkey ] = bool( inisetting.get( "settings", settingkey ) )

					else:
						settings_dict[ settingkey ] = inisetting.get( "settings", settingkey )

			for effectkey in [ "rotate", "horizontal-reverse", "vertical-reverse", "brightness", "contrast" ]:
				if inisetting.has_option( "effects", effectkey ):
					if effectkey in [ "horizontal-reverse", "vertical-reverse" ]:
						effects_dict[ effectkey ] = bool( inisetting.get( "effects", effectkey ) )
					else:
						effects_dict[ effectkey ] = int( inisetting.get( "effects", effectkey ) )

		if wx.DisplaySize()[1] < 960:
			config_dialog = IniEditDialog( "起動時のデフォルト設定", ( 1000, wx.DisplaySize()[1] - 50 ), settings_dict, effects_dict )
		else:
			config_dialog = IniEditDialog( "起動時のデフォルト設定", ( 1000, 920 ), settings_dict, effects_dict )

		#OKボタンを押してダイアログを閉じたら、設定の内容を組み上げてINIファイルとして保存する
		if config_dialog.ShowModal() == wx.ID_OK:
			edited_setting = config_dialog.get_configure_dictionary()
			edited_effect = config_dialog.get_effects_dictionary()
			ini_data = ""

			if edited_setting != {}:
				ini_data += "[settings]\n"

				if edited_setting.get( "window-mode", "" ) == "fullscreen" or edited_setting.get( "window-mode", "" ) == "window":
					ini_data += "window-mode = " + edited_setting.get( "window-mode" ) + "\n"

				if "window-size-x" in edited_setting and "window-size-y" in edited_setting:
					ini_data += "window-size = " + str( edited_setting.get( "window-size-x" ) ) + "x" + str( edited_setting.get( "window-size-y" ) ) + "\n"

				if "window-position-x" in edited_setting and "window-position-y" in edited_setting:
					ini_data += "window-position = " + str( edited_setting.get( "window-position-x" ) ) + "," + str( edited_setting.get( "window-position-y" ) ) + "\n"

				if edited_setting.get( "statusbar", False ) is True:
					ini_data += "statusbar = True\n"

				if edited_setting.get( "page-direction", "" ) == "RightToLeft":
					ini_data += "page-direction = RightToLeft\n"

				if "borderline-rate" in edited_setting:
					ini_data += "borderline-rate = " + str( edited_setting.get( "borderline-rate" ) ) + "\n"

				if "bgcolor-r" in edited_setting and "bgcolor-g" in edited_setting and "bgcolor-b" in edited_setting:
					ini_data += "background-color-rgb = " + str( edited_setting.get( "bgcolor-r" ) ) + "," + str( edited_setting.get( "bgcolor-g" ) ) + "," + str( edited_setting.get( "bgcolor-b" ) ) + "\n"

				if edited_setting.get( "show-extension", False ) is True:
					ini_data += "show-extension = True\n"

				if edited_setting.get( "drop-mode", "" ) != "":
					ini_data += "drop-mode = " + edited_setting.get( "drop-mode" ) + "\n"

				ini_data += "\n"

			if edited_effect != {}:
				ini_data += "[effects]\n"

				if "rotate" in edited_effect:
					ini_data += "rotate = " + str( edited_effect.get( "rotate" ) ) + "\n"

				if edited_effect.get( "horizontal-reverse", False ) is True:
					ini_data += "horizontal-reverse = True\n"

				if edited_effect.get( "vertical-reverse", False ) is True:
					ini_data += "vertical-reverse = True\n"

				if "brightness" in edited_effect:
					ini_data += "brightness = " + str( edited_effect.get( "brightness" ) ) + "\n"

				if "contrast" in edited_effect:
					ini_data += "contrast = " + str( edited_effect.get( "contrast" ) ) + "\n"

			f = open( reference_values.get_ini_path(), 'w' )
			f.write( ini_data )
			f.close()

		config_dialog.Destroy()

	#右クリックから現在表示中のタブを閉じる
	def rc_close_current_tab( self, event ):

		#タブがないときに閉じようとすると異常終了するので、タブ数が0を超える場合にのみ閉じる
		#とはいえ、ctrl+wで閉じるのと違い、タブ(パネル)が0ならこの右メニュー自体出ないが念の為
		if self.parent_object.GetPageCount() > 0:
			#ctrl+wのショートカットキーと違って、右クリックだと押し間違いかもしれないので
			#念の為に確認ダイアログを表示する
			question_dialog = wx.MessageDialog( None, "現在表示中のタブを閉じますか？", "確認", wx.YES_NO | wx.ICON_QUESTION )

			if question_dialog.ShowModal() == wx.ID_YES:
				self.parent_object.DeletePage( self.parent_object.GetSelection() )

			question_dialog.Destroy()


	#↑右クリックメニュー関連はここまで


	#画像ファイルのリストを更新する
	def reload_filelist( self, tab_number = -1 ):
		self.picture_file_list = []

		for i in maybe_filelist_natsort( os.listdir( self.directory ) ):
			filepath = os.path.join ( self.directory, i )

			if os.path.isfile( filepath ) :
				#画像のmimetypeになっているファイルのみをまとめる
				if re.match( "image/", str( mimetypes.guess_type( filepath )[0] ) ):
					self.picture_file_list.append(i)

		#更新した結果、画像が存在しなくなっていたらエラーメッセージを出す
		if self.picture_file_list == []:
			error_dialog =  wx.MessageDialog( None, "ディレクトリ「" + self.directory + "」に画像が存在していません", "Error",wx.OK | wx.ICON_ERROR )
			error_dialog.ShowModal()
			error_dialog.Destroy()
			return False

		#もしも再取得したファイルリストの中に現在表示しているファイルの名が
		#（ファイルが削除やりネームされていて）なければ、リストの最初の画像を現在位置とする。
		if self.current_picture_file not in self.picture_file_list:
			if tab_number < 0:
				tab_number = self.parent_object.GetSelection()
			error_dialog =  wx.MessageDialog( None, "更新した画像リストの中に、表示中のファイルが見つかりません。\nディレクトリの最初の画像を現在位置に設定し直します。\n\nタブ位置 ： " + str( tab_number + 1 ) + "／" + str( self.parent_object.GetPageCount() ) + "\n\n見つからなかったファイル名 ： " + self.current_picture_file, "Error",wx.OK | wx.ICON_ERROR )
			error_dialog.ShowModal()
			error_dialog.Destroy()
			self.current_picture_file = self.picture_file_list[0]

			#ファイル位置が移動したので画像表示を更新する
			self.page_move( 0 )

		elif "name2" in self.picture_infomations:
			#一枚目がリストにあっても、二枚並んだ表示で二枚目がリストになければ、表示を更新する
			if self.picture_infomations.get( 'name2' ) not in self.picture_file_list:
				self.page_move( 0 )


class TextDialog( wx.Dialog ):

	def __init__( self, title, size_tuple, message_text ):
		wx.Dialog.__init__( self, None, -1, title, size=size_tuple )

		self.text = wx.TextCtrl(self, style=wx.TE_MULTILINE | wx.TE_READONLY )
		self.text.write( message_text )
		button = wx.Button(self, wx.ID_OK, "OK")
		button.SetDefault()

		sizer = wx.BoxSizer( wx.VERTICAL )
		sizer.Add( self.text, 1, wx.LEFT | wx.RIGHT | wx.TOP | wx.EXPAND, 10 )
		sizer.Add(button, 0, wx.ALIGN_CENTER )
		self.SetSizer(sizer)


class SliderDialog( wx.Dialog ):

	def __init__( self, title, size_tuple, min_value, max_value, initial_value, message_text, list_title, destination_list ):
		wx.Dialog.__init__( self, None, -1, title, size=size_tuple )

		self.text = wx.TextCtrl(self, style=wx.TE_READONLY )
		self.text.write( message_text )

		self.slider = wx.Slider( self, style=wx.SL_HORIZONTAL | wx.SL_LABELS )
		self.slider.SetMin( min_value )
		self.slider.SetMax( max_value )
		self.slider.SetValue( initial_value )

		self.message_title = wx.StaticText( self, wx.ID_ANY, "＜" + list_title + "＞" )
		self.d_list = destination_list
		self.message_area = wx.StaticText( self, wx.ID_ANY, self.d_list[ initial_value - 1 ] )

		#スライダーを移動したらスライダー位置に対応するリストを表示するように関数へBindする
		self.slider.Bind( wx.EVT_SLIDER, self.extract_list )

		button = wx.Button(self, wx.ID_OK, "OK")
		button.SetDefault()

		sizer = wx.BoxSizer( wx.VERTICAL )
		sizer.Add( self.text, 0, wx.LEFT | wx.RIGHT | wx.EXPAND )
		sizer.Add( self.slider, 1, wx.ALL | wx.EXPAND, 10 )
		sizer.Add( wx.StaticLine( self ), flag=wx.GROW )
		sizer.Add( self.message_title, 1, wx.LEFT | wx.RIGHT | wx.TOP | wx.EXPAND, 20 )
		sizer.Add( self.message_area, 1, wx.LEFT | wx.RIGHT | wx.EXPAND, 30 )
		sizer.Add(button, 0, wx.ALIGN_CENTER )
		self.SetSizer(sizer)

	def extract_list( self, event ):
		self.message_area.SetLabel( self.d_list[ self.slider.GetValue() - 1 ] )

	def get_slider_value( self ):
		return self.slider.GetValue()


#整数・小数入力用に小規模な改造をしたテキスト入力
#wx.SpinCtrlは空欄にできないようなので、今回は没
class TextCtrlPlus( wx.TextCtrl ):

	def __init__( self, *args, **kwargs ):
		wx.TextCtrl.__init__( self, *args, **kwargs )

		#TextCtrlに非整数値を入力した場合に、空欄や直前の値に戻すことにしたが
		#そうすると、そのTextCtrlへの自動書き換え自体が入力と判定されてしまうようだ。
		#Bindした関数が再度呼びだされておかしなことになるため、管理用の目印を追加した。
		self.autoset_check = False

	def get_check( self ):
		return self.autoset_check

	def set_check( self, flag_bool ):
		self.autoset_check = flag_bool


class EffectsDialog( wx.Dialog ):

	def __init__( self, title, size_tuple, effects_dictionary, other_dictionary ):
		wx.Dialog.__init__( self, None, -1, title, size=size_tuple )

		self.effects = copy.deepcopy( effects_dictionary )
		self.other_values = copy.deepcopy( other_dictionary )

		#回転角度を操作するスライダー
		self.degree_slider = wx.Slider( self, style=wx.SL_HORIZONTAL | wx.SL_LABELS )
		self.degree_slider.SetMin( 0 )
		self.degree_slider.SetMax( 359 )
		if "rotate" in self.effects:
			self.degree_slider.SetValue( int( abs( self.effects.get( "rotate" ) ) % 360 ) )
		else:
			self.degree_slider.SetValue( 0 )

		#スライダーを移動したらdictionaryを書き換えるように関数へBindする
		self.degree_slider.Bind( wx.EVT_SLIDER, self.rotation_change )

		#90°ごとの回転にちょうど合わせるのが手間がかかるので、専用ボタンを用意する
		self.degree_0_button = wx.Button( self, wx.ID_ANY, "0°にする" )
		self.degree_90_button = wx.Button( self, wx.ID_ANY, "90°にする" )
		self.degree_180_button = wx.Button( self, wx.ID_ANY, "180°にする" )
		self.degree_270_button = wx.Button( self, wx.ID_ANY, "270°にする" )
		#ボタンを一行にする
		degree_buttons_sizer = wx.BoxSizer( wx.HORIZONTAL )
		degree_buttons_sizer.Add( self.degree_0_button, 0, wx.LEFT, 10 )
		degree_buttons_sizer.Add( self.degree_90_button, 0, wx.LEFT, 80 )
		degree_buttons_sizer.Add( self.degree_180_button, 0, wx.LEFT, 80 )
		degree_buttons_sizer.Add( self.degree_270_button, 0, wx.LEFT, 80 )

		#ボタンを押したらスライダーを対応した角度にするように関数へBindする
		self.degree_0_button.Bind( wx.EVT_BUTTON, self.set_slider_degree0 )
		self.degree_90_button.Bind( wx.EVT_BUTTON, self.set_slider_degree90 )
		self.degree_180_button.Bind( wx.EVT_BUTTON, self.set_slider_degree180 )
		self.degree_270_button.Bind( wx.EVT_BUTTON, self.set_slider_degree270 )

		#スライダーとボタンを枠（StaticBoxSizer）に入れる
		self.sbox_rotate = wx.StaticBox( self, wx.ID_ANY, "画像の回転（ 0°〜359° ）" )
		self.SB_sizer_rotate = wx.StaticBoxSizer( self.sbox_rotate, wx.VERTICAL )
		self.SB_sizer_rotate.Add( self.degree_slider, 1, wx.LEFT | wx.RIGHT | wx.EXPAND, 10 )
		self.SB_sizer_rotate.Add( degree_buttons_sizer, 1, wx.LEFT | wx.RIGHT | wx.EXPAND, 10 )

		#左右反転のチェックボックス
		self.checkbox_LR_reverse = wx.CheckBox( self, wx.ID_ANY, "画像の左右反転" )
		if "horizontal-reverse" in self.effects:
			self.checkbox_LR_reverse.SetValue( True )
		else:
			self.checkbox_LR_reverse.SetValue( False )

		#チェックボックスを変更したらdictionaryを書き換えるように関数へBindする
		self.checkbox_LR_reverse.Bind( wx.EVT_CHECKBOX, self.change_dict_LR )

		#チェックボックスを枠（StaticBoxSizer）に入れる
		self.sbox_LR_reverse = wx.StaticBox( self, wx.ID_ANY, "左右反転" )
		self.SB_sizer_LR_reverse = wx.StaticBoxSizer( self.sbox_LR_reverse, wx.VERTICAL )
		self.SB_sizer_LR_reverse.Add( self.checkbox_LR_reverse, 1, wx.LEFT | wx.RIGHT | wx.EXPAND, 50 )

		#上下反転のチェックボックス
		self.checkbox_TB_reverse = wx.CheckBox( self, wx.ID_ANY, "画像の上下反転" )
		if "vertical-reverse" in self.effects:
			self.checkbox_TB_reverse.SetValue( True )
		else:
			self.checkbox_TB_reverse.SetValue( False )

		#チェックボックスを変更したらdictionaryを書き換えるように関数へBindする
		self.checkbox_TB_reverse.Bind( wx.EVT_CHECKBOX, self.change_dict_TB )

		#チェックボックスを枠（StaticBoxSizer）に入れる
		self.sbox_TB_reverse = wx.StaticBox( self, wx.ID_ANY, "上下反転" )
		self.SB_sizer_TB_reverse = wx.StaticBoxSizer( self.sbox_TB_reverse, wx.VERTICAL )
		self.SB_sizer_TB_reverse.Add( self.checkbox_TB_reverse, 1, wx.LEFT | wx.RIGHT | wx.EXPAND, 50 )

		#このタブの境界値入力欄
		self.borderline_enter = TextCtrlPlus( self, wx.ID_ANY, str( self.other_values.get( "borderline-rate" ) ) )
		self.borderline_error_text = wx.StaticText(  self, wx.ID_ANY, "エラー表示欄　※常に一枚だけ表示させたいなら、通常ありえない0.01などにすれば擬似的に実現できます" )

		#境界比入力欄のBind
		self.borderline_enter.Bind( wx.EVT_TEXT, self.change_borderline_rate )

		#説明と入力欄などを、それぞれ1行にまとめる
		self.textline_borderline = wx.BoxSizer( wx.HORIZONTAL )
		self.textline_borderline.Add( wx.StaticText(  self, wx.ID_ANY, "画像の「幅／高さ」が2枚とも" ) )
		self.textline_borderline.Add( self.borderline_enter )
		self.textline_borderline.Add( wx.StaticText(  self, wx.ID_ANY, "未満だった場合に2枚並べて表示する　（少数値が可能です）" ) )

		#境界縦横比入力を枠（StaticBoxSizer）に入れる
		self.sbox_border = wx.StaticBox( self, wx.ID_ANY, "このタブで、画像を並べるか、1枚だけ表示するかどうかを分ける縦横比" )
		self.SB_sizer_border = wx.StaticBoxSizer( self.sbox_border, wx.VERTICAL )
		self.SB_sizer_border.Add( self.textline_borderline, 0, wx.LEFT | wx.RIGHT | wx.TOP, 10 )
		self.SB_sizer_border.Add( self.borderline_error_text, 0, wx.ALIGN_LEFT | wx.ALL, 10 )

		#画像の明度を操作するスライダー
		self.brightness_slider = wx.Slider( self, style=wx.SL_HORIZONTAL | wx.SL_LABELS )
		self.brightness_slider.SetMin( 0 )
		self.brightness_slider.SetMax( 200 )
		if "brightness" in self.effects:
			brightness_value = self.effects.get( "brightness" )
			if brightness_value >= 0 and brightness_value <= 200:
				self.brightness_slider.SetValue( int( brightness_value ) )
			else:
				self.brightness_slider.SetValue( 100 )
		else:
			self.brightness_slider.SetValue( 100 )

		#スライダーを移動したらdictionaryを書き換えるように関数へBindする
		self.brightness_slider.Bind( wx.EVT_SLIDER, self.brightness_change )

		#スライダーを枠（StaticBoxSizer）に入れる
		self.sbox_brightness = wx.StaticBox( self, wx.ID_ANY, "明るさ（％）　※100%で効果なし（元の画像のまま）になります" )
		self.SB_sizer_brightness = wx.StaticBoxSizer( self.sbox_brightness, wx.VERTICAL )
		self.SB_sizer_brightness.Add( self.brightness_slider, 1, wx.LEFT | wx.RIGHT | wx.EXPAND, 10 )

		#画像のコントラストを操作するスライダー
		self.contrast_slider = wx.Slider( self, style=wx.SL_HORIZONTAL | wx.SL_LABELS )
		self.contrast_slider.SetMin( 0 )
		self.contrast_slider.SetMax( 200 )
		if "contrast" in self.effects:
			contrast_value = self.effects.get( "contrast" )
			if contrast_value >= 0 and contrast_value <= 200:
				self.contrast_slider.SetValue( int( contrast_value ) )
			else:
				self.contrast_slider.SetValue( 100 )
		else:
			self.contrast_slider.SetValue( 100 )

		#スライダーを移動したらdictionaryを書き換えるように関数へBindする
		self.contrast_slider.Bind( wx.EVT_SLIDER, self.contrast_change )

		#スライダーを枠（StaticBoxSizer）に入れる
		self.sbox_contrast = wx.StaticBox( self, wx.ID_ANY, "コントラスト（％）　※100%で効果なし（元の画像のまま）になります" )
		self.SB_sizer_contrast = wx.StaticBoxSizer( self.sbox_contrast, wx.VERTICAL )
		self.SB_sizer_contrast.Add( self.contrast_slider, 1, wx.LEFT | wx.RIGHT | wx.EXPAND, 10 )

		ok_button = wx.Button( self, wx.ID_OK, "OK" )
		cancel_button = wx.Button( self, wx.ID_CANCEL, "Cancel" )
		twin_buttons_sizer = wx.BoxSizer( wx.HORIZONTAL )
		twin_buttons_sizer.Add( ok_button, 1, wx.LEFT | wx.RIGHT, 100 )
		twin_buttons_sizer.Add( cancel_button, 1, wx.LEFT | wx.RIGHT, 100 )

		#設定の枠とボタンをsizerでまとめる
		sizer = wx.BoxSizer( wx.VERTICAL )
		sizer.Add( self.SB_sizer_rotate, 1, wx.ALL | wx.EXPAND, 10 )
		sizer.Add( self.SB_sizer_LR_reverse, 1, wx.ALL | wx.EXPAND, 10 )
		sizer.Add( self.SB_sizer_TB_reverse, 1, wx.ALL | wx.EXPAND, 10 )
		sizer.Add( self.SB_sizer_border, 1, wx.ALL | wx.EXPAND, 10 )
		sizer.Add( self.SB_sizer_brightness, 1, wx.ALL | wx.EXPAND, 10 )
		sizer.Add( self.SB_sizer_contrast, 1, wx.ALL | wx.EXPAND, 10 )
		sizer.Add( twin_buttons_sizer, 0, wx.ALIGN_CENTER )
		self.SetSizer( sizer )

	#クラス内の設定dictionaryにおいて回転設定を書き換える
	def rotation_change( self, event ):
		degree = self.degree_slider.GetValue()
		if degree == 0:
			if  "rotate" in self.effects:
				del self.effects[ "rotate" ]
		else:
			self.effects[ "rotate" ] = degree

	#角度ボタンを押したら、対応した角度にスライダーと辞書の値を変更する
	def set_slider_degree0( self, event ):
		self.degree_slider.SetValue( 0 )
		if  "rotate" in self.effects:
			del self.effects[ "rotate" ]

	def set_slider_degree90( self, event ):
		self.degree_slider.SetValue( 90 )
		self.effects[ "rotate" ] = 90

	def set_slider_degree180( self, event ):
		self.degree_slider.SetValue( 180 )
		self.effects[ "rotate" ] = 180

	def set_slider_degree270( self, event ):
		self.degree_slider.SetValue( 270 )
		self.effects[ "rotate" ] = 270

	#クラス内の設定dictionaryにおいて左右反転の有無を書き換える
	def change_dict_LR( self, event ):
		if self.checkbox_LR_reverse.GetValue() is True:
			self.effects[ "horizontal-reverse" ] = 0
		else:
			del self.effects[ "horizontal-reverse" ]

	#クラス内の設定dictionaryにおいて上下反転の有無を書き換える
	def change_dict_TB( self, event ):
		if self.checkbox_TB_reverse.GetValue() is True:
			self.effects[ "vertical-reverse" ] = 0
		else:
			del self.effects[ "vertical-reverse" ]

	#境界縦横比入力の処理（整数値判定からの流用）
	def change_borderline_rate( self, event ):
		input_value = self.borderline_enter.GetValue()
		float_success = False

		try:
			self.other_values[ "borderline-rate" ] = float( input_value )
			float_success = True
		except:
			if input_value == "":
				self.borderline_enter.set_check( False )
			else:
				self.borderline_enter.SetValue( str( self.other_values.get( "borderline-rate" ) ) )
				self.borderline_enter.set_check( True )
				self.borderline_error_text.SetLabel( "境界縦横比の入力値が小数値に変換できなかったので、以前の値に戻しました（場合によっては空白化）" )

		if float_success is True:
			if not self.borderline_enter.get_check():
				self.borderline_error_text.SetLabel( "" )
			self.borderline_enter.set_check( False )

	def change_show_extension( self, event ):
		if self.checkbox_extension.GetValue() is True:
			self.config_dict[ "show-extension" ] = True
		else:
			if "show-extension" in self.config_dict:
				del self.config_dict[ "show-extension" ]

	#クラス内の設定dictionaryにおいて明るさの設定を書き換える
	def brightness_change( self, event ):
		brightness_value = self.brightness_slider.GetValue()
		if brightness_value == 100:
			if  "brightness" in self.effects:
				del self.effects[ "brightness" ]
		else:
			self.effects[ "brightness" ] = brightness_value

	#クラス内の設定dictionaryにおいてコントラストの設定を書き換える
	def contrast_change( self, event ):
		contrast_value = self.contrast_slider.GetValue()
		if contrast_value == 100:
			if  "contrast" in self.effects:
				del self.effects[ "contrast" ]
		else:
			self.effects[ "contrast" ] = contrast_value

	def get_effects_dictionary( self ):
		return self.effects

	def get_other_dictionary( self ):
		return self.other_values


class IniEditDialog( wx.Dialog ):

	def __init__( self, title, size_tuple, settings_dict, effects_dict ):
		wx.Dialog.__init__( self, None, -1, title, size=size_tuple )
		self.config_dict = copy.deepcopy( settings_dict )
		self.effect_dict = copy.deepcopy( effects_dict )

		notebook = wx.Notebook( self, wx.ID_ANY )

		setting_tab_panel = wx.lib.scrolledpanel.ScrolledPanel( notebook, wx.ID_ANY )
		effect_tab_panel = wx.lib.scrolledpanel.ScrolledPanel( notebook, wx.ID_ANY )
		notebook.AddPage( setting_tab_panel, '本体設定' )
		notebook.AddPage( effect_tab_panel, '画像エフェクト' )

		#以下、本体設定タブの内容

		#ウィンドウ関連（モード、サイズ、位置）の構築
		windowmode_text = wx.StaticText( setting_tab_panel, wx.ID_ANY, "起動時のモード（最大化 or 全画面 or ウィンドウ）" )
		mode_array = ( "最大化", "全画面", "通常ウィンドウ" )
		self.window_mode_combobox = wx.ComboBox( setting_tab_panel, wx.ID_ANY, "モード", choices=mode_array, style=wx.CB_READONLY )
		if self.config_dict.get( "window-mode", "" ) == "fullscreen":
			self.window_mode_combobox.SetStringSelection( "全画面" )
		elif self.config_dict.get( "window-mode", "" ) == "window":
			self.window_mode_combobox.SetStringSelection( "通常ウィンドウ" )
		else:
			self.window_mode_combobox.SetStringSelection( "最大化" )
		windowsize_text1 = wx.StaticText( setting_tab_panel, wx.ID_ANY, "ウィンドウサイズ　幅：" )
		self.size_enter_x = TextCtrlPlus( setting_tab_panel, wx.ID_ANY, str( self.config_dict.get( "window-size-x", "" ) ) )
		windowsize_text2 = wx.StaticText( setting_tab_panel, wx.ID_ANY, "、高さ：" )
		self.size_enter_y = TextCtrlPlus( setting_tab_panel, wx.ID_ANY, str( self.config_dict.get( "window-size-y", "" ) ) )
		window_position_text1 = wx.StaticText( setting_tab_panel, wx.ID_ANY, " ・ ウィンドウ位置　左上から （" )
		self.position_enter_x = TextCtrlPlus( setting_tab_panel, wx.ID_ANY, str( self.config_dict.get( "window-position-x", "" ) ) )
		window_position_text2 = wx.StaticText( setting_tab_panel, wx.ID_ANY, "　，" )
		self.position_enter_y = TextCtrlPlus( setting_tab_panel, wx.ID_ANY, str( self.config_dict.get( "window-position-y", "" ) ) )
		window_position_text3 = wx.StaticText( setting_tab_panel, wx.ID_ANY, "）" )
		self.window_error_text = wx.StaticText( setting_tab_panel, wx.ID_ANY, "入力値にエラーがあれば、ここに表示されます" )

		#各入力欄のBind
		self.window_mode_combobox.Bind( wx.EVT_COMBOBOX, self.change_window_mode )
		self.size_enter_x.Bind( wx.EVT_TEXT, self.change_window_width )
		self.size_enter_y.Bind( wx.EVT_TEXT, self.change_window_height )
		self.position_enter_x.Bind( wx.EVT_TEXT, self.change_window_position_x )
		self.position_enter_y.Bind( wx.EVT_TEXT, self.change_window_position_y )

		#項目名と入力欄などを、それぞれ1行にまとめる
		textline_winmode = wx.BoxSizer( wx.HORIZONTAL )
		textline_winmode.Add( windowmode_text )
		textline_winmode.Add( self.window_mode_combobox )
		textline_win_size_position = wx.BoxSizer( wx.HORIZONTAL )
		textline_win_size_position.Add( windowsize_text1 )
		textline_win_size_position.Add( self.size_enter_x )
		textline_win_size_position.Add( windowsize_text2 )
		textline_win_size_position.Add( self.size_enter_y )
		textline_win_size_position.Add( window_position_text1 )
		textline_win_size_position.Add( self.position_enter_x )
		textline_win_size_position.Add( window_position_text2 )
		textline_win_size_position.Add( self.position_enter_y )
		textline_win_size_position.Add( window_position_text3 )
		#1行にまとまったところで、縦に並べる
		winsize_sizer = wx.BoxSizer( wx.VERTICAL )
		winsize_sizer.Add( textline_winmode, 0, wx.LEFT | wx.RIGHT | wx.TOP, 10 )
		winsize_sizer.Add( textline_win_size_position, 0, wx.ALL, 4 )
		winsize_sizer.Add( self.window_error_text, 0, wx.ALIGN_LEFT | wx.ALL, 10 )

		#ウィンドウサイズ指定を枠（StaticBoxSizer）に入れる
		sbox1 = wx.StaticBox( setting_tab_panel, wx.ID_ANY, "ウィンドウのモード、通常ウィンドウ時のサイズおよび位置" )
		SB_sizer1 = wx.StaticBoxSizer( sbox1, wx.VERTICAL )
		SB_sizer1.Add( winsize_sizer, 1, wx.LEFT | wx.RIGHT | wx.EXPAND, 10 )

		#ステータスバー
		self.checkbox_statusbar = wx.CheckBox( setting_tab_panel, wx.ID_ANY, "ステータスバーを表示する" )
		if "statusbar" in self.config_dict:
			self.checkbox_statusbar.SetValue( True )
		else:
			self.checkbox_statusbar.SetValue( False )

		#ステータスバーcheckboxのBind
		self.checkbox_statusbar.Bind( wx.EVT_CHECKBOX, self.change_showstatusbar )

		#ステータスバーの表示チェックを枠（StaticBoxSizer）に入れる
		sbox2 = wx.StaticBox( setting_tab_panel, wx.ID_ANY, "ステータスバー" )
		SB_sizer2 = wx.StaticBoxSizer( sbox2, wx.VERTICAL )
		SB_sizer2.Add( self.checkbox_statusbar, 0, wx.ALL, 10 )

		#並べる方向
		direction_array = ( "左から右", "右から左" )
		self.direction_combobox = wx.ComboBox( setting_tab_panel, wx.ID_ANY, "方向", choices=direction_array, style=wx.CB_READONLY )
		direction_text = wx.StaticText( setting_tab_panel, wx.ID_ANY, "の順に画像を並べる （左から右だと、それがデフォルトなのでINI中の記述は消えます）" )
		if self.config_dict.get( "page-direction", "" ) == "RightToLeft":
			self.direction_combobox.SetStringSelection( "右から左" )
		else:
			self.direction_combobox.SetStringSelection( "左から右" )

		#画像方向comboboxのBind
		self.direction_combobox.Bind( wx.EVT_COMBOBOX, self.change_page_direction )

		textline_direction = wx.BoxSizer( wx.HORIZONTAL )
		textline_direction.Add( self.direction_combobox )
		textline_direction.Add( direction_text )

		#画像方向指定を枠（StaticBoxSizer）に入れる
		sbox3 = wx.StaticBox( setting_tab_panel, wx.ID_ANY, "縦長の画像を並べる方向" )
		SB_sizer3 = wx.StaticBoxSizer( sbox3, wx.VERTICAL )
		SB_sizer3.Add( textline_direction, 0, wx.ALL, 10 )

		#画像を並べるかどうかの境界縦横比
		borderline_text1 = wx.StaticText( setting_tab_panel, wx.ID_ANY, "画像の「幅／高さ」が2枚とも" )
		self.borderline_enter = TextCtrlPlus( setting_tab_panel, wx.ID_ANY, str( self.config_dict.get( "borderline-rate", "" ) ) )
		borderline_text2 = wx.StaticText( setting_tab_panel, wx.ID_ANY, "未満だった場合に2枚並べて表示する　（少数値が可能です）" )
		self.borderline_error_text = wx.StaticText( setting_tab_panel, wx.ID_ANY, "エラー表示欄　※常に一枚だけ表示させたいなら、通常ありえない0.01などにすれば擬似的に実現できます" )

		#境界比入力欄のBind
		self.borderline_enter.Bind( wx.EVT_TEXT, self.change_borderline_rate )

		#説明と入力欄などを、それぞれ1行にまとめる
		textline_borderline = wx.BoxSizer( wx.HORIZONTAL )
		textline_borderline.Add( borderline_text1 )
		textline_borderline.Add( self.borderline_enter )
		textline_borderline.Add( borderline_text2 )

		#境界縦横比入力を枠（StaticBoxSizer）に入れる
		sbox4 = wx.StaticBox( setting_tab_panel, wx.ID_ANY, "画像を並べるか、1枚だけ表示するかどうかの境界縦横比" )
		SB_sizer4 = wx.StaticBoxSizer( sbox4, wx.VERTICAL )
		SB_sizer4.Add( textline_borderline, 0, wx.LEFT | wx.RIGHT | wx.TOP, 10 )
		SB_sizer4.Add( self.borderline_error_text, 0, wx.ALIGN_LEFT | wx.ALL, 10 )

		#背景色
		bgcolor_text1 = wx.StaticText( setting_tab_panel, wx.ID_ANY, "赤（R）：" )
		self.bgcolor_enter1 = TextCtrlPlus( setting_tab_panel, wx.ID_ANY, str( self.config_dict.get( "bgcolor-r", "" ) ) )
		bgcolor_text2 = wx.StaticText( setting_tab_panel, wx.ID_ANY, "、緑（G）：" )
		self.bgcolor_enter2 = TextCtrlPlus( setting_tab_panel, wx.ID_ANY, str( self.config_dict.get( "bgcolor-g", "" ) ) )
		bgcolor_text3 = wx.StaticText( setting_tab_panel, wx.ID_ANY, "、青（B）：" )
		self.bgcolor_enter3 = TextCtrlPlus( setting_tab_panel, wx.ID_ANY, str( self.config_dict.get( "bgcolor-b", "" ) ) )
		self.bgcolor_error_text = wx.StaticText( setting_tab_panel, wx.ID_ANY, "入力値にエラーがあれば、ここに表示されます" )

		#各入力欄のBind
		self.bgcolor_enter1.Bind( wx.EVT_TEXT, self.change_bgcolor_r )
		self.bgcolor_enter2.Bind( wx.EVT_TEXT, self.change_bgcolor_g )
		self.bgcolor_enter3.Bind( wx.EVT_TEXT, self.change_bgcolor_b )

		#説明と入力欄などを、それぞれ1行にまとめる
		textline_bgcolor = wx.BoxSizer( wx.HORIZONTAL )
		textline_bgcolor.Add( bgcolor_text1 )
		textline_bgcolor.Add( self.bgcolor_enter1 )
		textline_bgcolor.Add( bgcolor_text2 )
		textline_bgcolor.Add( self.bgcolor_enter2 )
		textline_bgcolor.Add( bgcolor_text3 )
		textline_bgcolor.Add( self.bgcolor_enter3 )

		#背景色指定を枠（StaticBoxSizer）に入れる
		sbox5 = wx.StaticBox( setting_tab_panel, wx.ID_ANY, "背景色の指定" )
		SB_sizer5 = wx.StaticBoxSizer( sbox5, wx.VERTICAL )
		SB_sizer5.Add( textline_bgcolor, 0, wx.ALL, 10 )
		SB_sizer5.Add( self.bgcolor_error_text, 0, wx.ALIGN_LEFT | wx.ALL, 10 )

		#タブの拡張子
		self.checkbox_extension = wx.CheckBox( setting_tab_panel, wx.ID_ANY, "タブにファイルの拡張子を表示する" )
		if "show-extension" in self.config_dict:
			self.checkbox_extension.SetValue( True )
		else:
			self.checkbox_extension.SetValue( False )

		#拡張子checkboxのBind
		self.checkbox_extension.Bind( wx.EVT_CHECKBOX, self.change_show_extension )

		#拡張子の表示チェックを枠（StaticBoxSizer）に入れる
		sbox6 = wx.StaticBox( setting_tab_panel, wx.ID_ANY, "タブ中の拡張子表示" )
		SB_sizer6 = wx.StaticBoxSizer( sbox6, wx.VERTICAL )
		SB_sizer6.Add( self.checkbox_extension, 0, wx.ALL, 10 )

		#ドラッグ＆ドロップの対応
		dragdrop_array = ( "右端に開く", "現在のタブの右に開く", "ダイアログで尋ねる" )
		self.dragdrop_combobox = wx.ComboBox( setting_tab_panel, wx.ID_ANY, "モード", choices=dragdrop_array, style=wx.CB_READONLY )
		if self.config_dict.get( "drop-mode", "" ) == "current-right":
			self.dragdrop_combobox.SetStringSelection( "現在のタブの右に開く" )
		elif self.config_dict.get( "drop-mode", "" ) == "ask":
			self.dragdrop_combobox.SetStringSelection( "ダイアログで尋ねる" )
		else:
			self.dragdrop_combobox.SetStringSelection( "右端に開く" )

		#ドラッグ＆ドロップcomboboxのBind
		self.dragdrop_combobox.Bind( wx.EVT_COMBOBOX, self.change_dragdrop_mode )

		#ドラッグ＆ドロップcomboboxを枠（StaticBoxSizer）に入れる
		sbox7 = wx.StaticBox( setting_tab_panel, wx.ID_ANY, "ドラッグ＆ドロップされた画像ファイルの対応 （リストファイルなら、そこに記された画像ファイルをどうするか）" )
		SB_sizer7 = wx.StaticBoxSizer( sbox7, wx.VERTICAL )
		SB_sizer7.Add( self.dragdrop_combobox, 0, wx.ALL, 10 )

		setting_tab_sizer = wx.BoxSizer( wx.VERTICAL )
		setting_tab_sizer.Add( SB_sizer1, 0, wx.LEFT | wx.RIGHT | wx.TOP | wx.EXPAND, 10 )
		setting_tab_sizer.Add( SB_sizer2, 0, wx.LEFT | wx.RIGHT | wx.TOP | wx.EXPAND, 10 )
		setting_tab_sizer.Add( SB_sizer3, 0, wx.LEFT | wx.RIGHT | wx.TOP | wx.EXPAND, 10 )
		setting_tab_sizer.Add( SB_sizer4, 0, wx.LEFT | wx.RIGHT | wx.TOP | wx.EXPAND, 10 )
		setting_tab_sizer.Add( SB_sizer5, 0, wx.LEFT | wx.RIGHT | wx.TOP | wx.EXPAND, 10 )
		setting_tab_sizer.Add( SB_sizer6, 0, wx.LEFT | wx.RIGHT | wx.TOP | wx.EXPAND, 10 )
		setting_tab_sizer.Add( SB_sizer7, 0, wx.LEFT | wx.RIGHT | wx.TOP | wx.EXPAND, 10 )
		setting_tab_panel.SetSizer( setting_tab_sizer )


		#以下、エフェクトタブの内容（エフェクトダイアログからの流用）

		#回転角度を操作するスライダー
		self.degree_slider = wx.Slider( effect_tab_panel, style=wx.SL_HORIZONTAL | wx.SL_LABELS )
		self.degree_slider.SetMin( 0 )
		self.degree_slider.SetMax( 360 )
		if "rotate" in self.effect_dict:
			self.degree_slider.SetValue( int( abs( self.effect_dict.get( "rotate" ) ) % 360 ) )
		else:
			self.degree_slider.SetValue( 0 )

		#回転角度変更を関数へBindする
		self.degree_slider.Bind( wx.EVT_SLIDER, self.rotation_change )

		#90°ごとの回転に専用ボタンを用意する
		self.degree_0_button = wx.Button( effect_tab_panel, wx.ID_ANY, "0°にする" )
		self.degree_90_button = wx.Button( effect_tab_panel, wx.ID_ANY, "90°にする" )
		self.degree_180_button = wx.Button( effect_tab_panel, wx.ID_ANY, "180°にする" )
		self.degree_270_button = wx.Button( effect_tab_panel, wx.ID_ANY, "270°にする" )
		#ボタンを一行にする
		degree_buttons_sizer = wx.BoxSizer( wx.HORIZONTAL )
		degree_buttons_sizer.Add( self.degree_0_button, 0, wx.LEFT, 10 )
		degree_buttons_sizer.Add( self.degree_90_button, 0, wx.LEFT, 80 )
		degree_buttons_sizer.Add( self.degree_180_button, 0, wx.LEFT, 80 )
		degree_buttons_sizer.Add( self.degree_270_button, 0, wx.LEFT, 80 )

		#ボタンを押したらスライダーを対応した角度にするように関数へBindする
		self.degree_0_button.Bind( wx.EVT_BUTTON, self.set_slider_degree0 )
		self.degree_90_button.Bind( wx.EVT_BUTTON, self.set_slider_degree90 )
		self.degree_180_button.Bind( wx.EVT_BUTTON, self.set_slider_degree180 )
		self.degree_270_button.Bind( wx.EVT_BUTTON, self.set_slider_degree270 )

		#スライダーとボタンを枠（StaticBoxSizer）に入れる
		sbox_e1 = wx.StaticBox( effect_tab_panel, wx.ID_ANY, "画像の回転（ 0°〜359° ）　※0°では効果無しなのでINI中の記述は消えます" )
		SB_sizer_e1 = wx.StaticBoxSizer( sbox_e1, wx.VERTICAL )
		SB_sizer_e1.Add( self.degree_slider, 1, wx.ALL | wx.EXPAND, 10 )
		SB_sizer_e1.Add( degree_buttons_sizer, 0, wx.LEFT | wx.RIGHT | wx.EXPAND, 10 )

		self.checkbox_LR_reverse = wx.CheckBox( effect_tab_panel, wx.ID_ANY, "画像の左右反転" )
		if "horizontal-reverse" in self.effect_dict:
			self.checkbox_LR_reverse.SetValue( True )
		else:
			self.checkbox_LR_reverse.SetValue( False )

		#左右反転の変更を関数へBindする
		self.checkbox_LR_reverse.Bind( wx.EVT_CHECKBOX, self.change_dict_LR )

		#チェックボックスを枠（StaticBoxSizer）に入れる
		sbox_e2 = wx.StaticBox( effect_tab_panel, wx.ID_ANY, "左右反転" )
		SB_sizer_e2 = wx.StaticBoxSizer( sbox_e2, wx.VERTICAL )
		SB_sizer_e2.Add( self.checkbox_LR_reverse, 0, wx.ALL, 20 )

		self.checkbox_TB_reverse = wx.CheckBox( effect_tab_panel, wx.ID_ANY, "画像の上下反転" )
		if "vertical-reverse" in self.effect_dict:
			self.checkbox_TB_reverse.SetValue( True )
		else:
			self.checkbox_TB_reverse.SetValue( False )

		#上下反転を関数へBindする
		self.checkbox_TB_reverse.Bind( wx.EVT_CHECKBOX, self.change_dict_TB )

		#チェックボックスを枠（StaticBoxSizer）に入れる
		sbox_e3 = wx.StaticBox( effect_tab_panel, wx.ID_ANY, "上下反転" )
		SB_sizer_e3 = wx.StaticBoxSizer( sbox_e3, wx.VERTICAL )
		SB_sizer_e3.Add( self.checkbox_TB_reverse, 0, wx.ALL, 20 )

		#画像の明度を操作するスライダー
		self.brightness_slider = wx.Slider( effect_tab_panel, style=wx.SL_HORIZONTAL | wx.SL_LABELS )
		self.brightness_slider.SetMin( 0 )
		self.brightness_slider.SetMax( 200 )
		if "brightness" in self.effect_dict:
			brightness_value = self.effect_dict.get( "brightness" )
			if brightness_value >= 0 and brightness_value <= 200:
				self.brightness_slider.SetValue( int( brightness_value ) )
			else:
				self.brightness_slider.SetValue( 100 )
		else:
			self.brightness_slider.SetValue( 100 )

		#画像の明るさ変更を関数へBindする
		self.brightness_slider.Bind( wx.EVT_SLIDER, self.brightness_change )

		#スライダーを枠（StaticBoxSizer）に入れる
		sbox_e4 = wx.StaticBox( effect_tab_panel, wx.ID_ANY, "明るさ（％）　※100%で効果なし（元の画像をそのまま表示）になります" )
		SB_sizer_e4 = wx.StaticBoxSizer( sbox_e4, wx.VERTICAL )
		SB_sizer_e4.Add( self.brightness_slider, 1, wx.ALL | wx.EXPAND, 10 )

		#画像のコントラストを操作するスライダー
		self.contrast_slider = wx.Slider( effect_tab_panel, style=wx.SL_HORIZONTAL | wx.SL_LABELS )
		self.contrast_slider.SetMin( 0 )
		self.contrast_slider.SetMax( 200 )
		if "contrast" in self.effect_dict:
			contrast_value = self.effect_dict.get( "contrast" )
			if contrast_value >= 0 and contrast_value <= 200:
				self.contrast_slider.SetValue( int( contrast_value ) )
			else:
				self.contrast_slider.SetValue( 100 )
		else:
			self.contrast_slider.SetValue( 100 )

		#コントラスト変更を関数へBindする
		self.contrast_slider.Bind( wx.EVT_SLIDER, self.contrast_change )

		#スライダーを枠（StaticBoxSizer）に入れる
		sbox_e5 = wx.StaticBox( effect_tab_panel, wx.ID_ANY, "コントラスト（％）　※100%で効果なし（元の画像をそのまま表示）になります" )
		SB_sizer_e5 = wx.StaticBoxSizer( sbox_e5, wx.VERTICAL )
		SB_sizer_e5.Add( self.contrast_slider, 1, wx.ALL | wx.EXPAND, 10 )

		effect_tab_sizer = wx.BoxSizer( wx.VERTICAL )
		effect_tab_sizer.Add( SB_sizer_e1, 0, wx.LEFT | wx.RIGHT | wx.TOP | wx.EXPAND, 10 )
		effect_tab_sizer.Add( SB_sizer_e2, 0, wx.LEFT | wx.RIGHT | wx.TOP | wx.EXPAND, 10 )
		effect_tab_sizer.Add( SB_sizer_e3, 0, wx.LEFT | wx.RIGHT | wx.TOP | wx.EXPAND, 10 )
		effect_tab_sizer.Add( SB_sizer_e4, 0, wx.LEFT | wx.RIGHT | wx.TOP | wx.EXPAND, 10 )
		effect_tab_sizer.Add( SB_sizer_e5, 0, wx.LEFT | wx.RIGHT | wx.TOP | wx.EXPAND, 10 )
		effect_tab_panel.SetSizer( effect_tab_sizer )


		#以下、ダイアログ全体の構築

		config_notice1 = wx.StaticText( self, wx.ID_ANY, "※ これは起動時の設定の編集であって、現在の状態は変更しません" )
		config_notice2 = wx.StaticText( self, wx.ID_ANY, "入力欄が空、チェックがないといった場合は、デフォルト値の適用となりINIファイル中の記述は消えます" )

		ok_button = wx.Button( self, wx.ID_OK, "OK" )
		cancel_button = wx.Button( self, wx.ID_CANCEL, "Cancel" )
		twin_buttons_sizer = wx.BoxSizer( wx.HORIZONTAL )
		twin_buttons_sizer.Add( ok_button, 1, wx.LEFT | wx.RIGHT, 100 )
		twin_buttons_sizer.Add( cancel_button, 1, wx.LEFT | wx.RIGHT, 100 )

		#注意書きとタブとOK,Cancelボタンを縦に並べる
		sizer = wx.BoxSizer( wx.VERTICAL )
		sizer.Add( notebook, 1, wx.ALL | wx.EXPAND )
		sizer.Add( config_notice1, 0, wx.ALIGN_CENTER )
		sizer.Add( config_notice2, 0, wx.ALIGN_CENTER )
		sizer.Add( twin_buttons_sizer, 0, wx.ALIGN_CENTER )
		self.SetSizer( sizer )
		notebook.Refresh()

	def get_settings( self ):
		return self.config_dict

	def get_effects( self ):
		return self.effect_dict

	def change_window_mode( self, event ):
		choice = self.window_mode_combobox.GetSelection()
		if choice == 1:
			self.config_dict[ "window-mode" ] = "fullscreen"
		elif choice == 2:
			self.config_dict[ "window-mode" ] = "window"
		else:
			if "window-mode" in self.config_dict:
					del self.config_dict[ "window-mode" ]

	#以下、整数値の入力欄をまとめて書く
	def change_window_width( self, event ):
		self.textctrl_integer_check( self.size_enter_x, self.config_dict, "window-size-x", self.window_error_text, "ウィンドウの幅" )
	def change_window_height( self, event ):
		self.textctrl_integer_check( self.size_enter_y, self.config_dict, "window-size-y", self.window_error_text, "ウィンドウの高さ" )
	def change_window_position_x( self, event ):
		self.textctrl_integer_check( self.position_enter_x, self.config_dict, "window-position-x", self.window_error_text, "ウィンドウX座標" )
	def change_window_position_y( self, event ):
		self.textctrl_integer_check( self.position_enter_y, self.config_dict, "window-position-y", self.window_error_text, "ウィンドウY座標" )
	def change_bgcolor_r( self, event ):
		self.textctrl_integer_check( self.bgcolor_enter1, self.config_dict, "bgcolor-r", self.bgcolor_error_text, "背景色（赤）" )
	def change_bgcolor_g( self, event ):
		self.textctrl_integer_check( self.bgcolor_enter2, self.config_dict, "bgcolor-g", self.bgcolor_error_text, "背景色（緑）" )
	def change_bgcolor_b( self, event ):
		self.textctrl_integer_check( self.bgcolor_enter3, self.config_dict, "bgcolor-b", self.bgcolor_error_text, "背景色（青）" )

	#入力欄の値が整数値かどうか判定する
	def textctrl_integer_check( self, input_instance, dictionary, key_string, errormessage_instance, error_label ):
		input_value = input_instance.GetValue()
		integer_success = False

		try:
			dictionary[ key_string ] = int( input_instance.GetValue() )
			integer_success = True
		except:
			if input_value == "":
				if key_string in dictionary:
					del dictionary[ key_string ]
				input_instance.set_check( False )
			else:
				input_instance.SetValue( str( dictionary.get( key_string, "" ) ) )
				input_instance.set_check( True )
				errormessage_instance.SetLabel( error_label + "の入力値が整数値に変換できなかったので、以前の値に戻しました（場合によっては空白化）" )

		#ここらへんの処理は、自分でもなにがなんだかわからない。
		#非整数値の手動入力や貼り付けで不可解な流れになるため、場当たり的に対応した結果。
		if integer_success:
			if not input_instance.get_check():
				errormessage_instance.SetLabel( "" )
			input_instance.set_check( False )

	def change_showstatusbar( self, event ):
		if self.checkbox_statusbar.GetValue() is True:
			self.config_dict[ "statusbar" ] = True
		else:
			if "statusbar" in self.config_dict:
				del self.config_dict[ "statusbar" ]

	def change_page_direction( self, event ):
		choice = self.direction_combobox.GetSelection()
		if choice == 1:
			self.config_dict[ "page-direction" ] = "RightToLeft"
		else:
			if "page-direction" in self.config_dict:
					del self.config_dict[ "page-direction" ]

	#境界縦横比入力の処理（整数値判定からの流用）
	def change_borderline_rate( self, event ):
		input_value = self.borderline_enter.GetValue()
		float_success = False

		try:
			self.config_dict[ "borderline-rate" ] = float( self.borderline_enter.GetValue() )
			float_success = True
		except:
			if input_value == "":
				if "borderline-rate" in self.config_dict:
					del self.config_dict[ "borderline-rate" ]
				self.borderline_enter.set_check( False )
			else:
				self.borderline_enter.SetValue( str( self.config_dict.get( "borderline-rate", "" ) ) )
				self.borderline_enter.set_check( True )
				self.borderline_error_text.SetLabel( "境界縦横比の入力値が小数値に変換できなかったので、以前の値に戻しました（場合によっては空白化）" )

		if float_success is True:
			if not self.borderline_enter.get_check():
				self.borderline_error_text.SetLabel( "" )
			self.borderline_enter.set_check( False )

	def change_show_extension( self, event ):
		if self.checkbox_extension.GetValue() is True:
			self.config_dict[ "show-extension" ] = True
		else:
			if "show-extension" in self.config_dict:
				del self.config_dict[ "show-extension" ]

	def change_dragdrop_mode( self, event ):
		if self.dragdrop_combobox.GetSelection() == 0:
			if "drop-mode" in self.config_dict:
				del self.config_dict[ "drop-mode" ]
		elif self.dragdrop_combobox.GetSelection() == 1:
			self.config_dict[ "drop-mode" ] = "current-right"
		else:
			self.config_dict[ "drop-mode" ] = "ask"

	#クラス内の設定dictionaryにおいて回転設定を書き換える
	def rotation_change( self, event ):
		degree = self.degree_slider.GetValue()
		if degree == 0:
			if "rotate" in self.effect_dict:
				del self.effect_dict[ "rotate" ]
		else:
			self.effect_dict[ "rotate" ] = degree

	#角度ボタンを押したら、対応した角度にスライダーと辞書の値を変更する
	def set_slider_degree0( self, event ):
		self.degree_slider.SetValue( 0 )
		if "rotate" in self.effect_dict:
			del self.effect_dict[ "rotate" ]

	def set_slider_degree90( self, event ):
		self.degree_slider.SetValue( 90 )
		self.effect_dict[ "rotate" ] = 90

	def set_slider_degree180( self, event ):
		self.degree_slider.SetValue( 180 )
		self.effect_dict[ "rotate" ] = 180

	def set_slider_degree270( self, event ):
		self.degree_slider.SetValue( 270 )
		self.effect_dict[ "rotate" ] = 270

	#クラス内の設定dictionaryにおいて左右反転の有無を書き換える
	def change_dict_LR( self, event ):
		if self.checkbox_LR_reverse.GetValue() is True:
			self.effect_dict[ "horizontal-reverse" ] = True
		else:
			if "horizontal-reverse" in self.effect_dict:
				del self.effect_dict[ "horizontal-reverse" ]

	#クラス内の設定dictionaryにおいて上下反転の有無を書き換える
	def change_dict_TB( self, event ):
		if self.checkbox_TB_reverse.GetValue() is True:
			self.effect_dict[ "vertical-reverse" ] = True
		else:
			if "vertical-reverse" in self.effect_dict:
				del self.effect_dict[ "vertical-reverse" ]

	#クラス内の設定dictionaryにおいて明るさの設定を書き換える
	def brightness_change( self, event ):
		brightness_value = self.brightness_slider.GetValue()
		if brightness_value == 100:
			if  "brightness" in self.effect_dict:
				del self.effect_dict[ "brightness" ]
		else:
			self.effect_dict[ "brightness" ] = brightness_value

	#クラス内の設定dictionaryにおいてコントラストの設定を書き換える
	def contrast_change( self, event ):
		contrast_value = self.contrast_slider.GetValue()
		if contrast_value == 100:
			if  "contrast" in self.effect_dict:
				del self.effect_dict[ "contrast" ]
		else:
			self.effect_dict[ "contrast" ] = contrast_value

	def get_configure_dictionary( self ):
		return self.config_dict

	def get_effects_dictionary( self ):
		return self.effect_dict


#各タブごとの設定の大本となる基本設定、およびエラーになったファイルのリスト
#最初はextended_frameに含めていたが、肥大化してきたので分離した。
class BaseValue_and_Errorlist():

	def __init__( self ):
		self.borderline_rate = 1.0
		self.picture_bgcolor = [ 180, 180, 180 ]
		self.base_page_direction = "LeftToRight"
		self.file_existence_errorlist = []
		self.fileread_errorlist = []
		self.show_extension = False
		self.base_effects = {}
		self.ini_path = ""
		self.default_directory = ""
		self.drop_mode = ""

		#背景用WX画像をディスプレイサイズで用意する。（背景色を指定するため）
		self.background_wximage = wx.Image( wx.DisplaySize()[0], wx.DisplaySize()[1] )
		self.background_wximage.SetRGB( wx.Rect( 0, 0, self.background_wximage.GetWidth(), self.background_wximage.GetHeight() ), self.picture_bgcolor[0], self.picture_bgcolor[1], self.picture_bgcolor[2] )

	def set_borderline_rate( self, arg_rate ):
		self.borderline_rate = arg_rate

	def get_borderline_rate( self ):
		return self.borderline_rate

	def set_bgcolor( self, color_list ):
		self.picture_bgcolor = color_list
		self.background_wximage.SetRGB( wx.Rect( 0, 0, self.background_wximage.GetWidth(), self.background_wximage.GetHeight() ), self.picture_bgcolor[0], self.picture_bgcolor[1], self.picture_bgcolor[2] )

	def get_bgcolor( self ):
		return self.picture_bgcolor

	def set_page_direction( self, arg_direction ):
		self.base_page_direction = arg_direction

	def get_page_direction( self ):
		return self.base_page_direction

	def add_existence_error( self, errorfile ):
		if isinstance( errorfile, list ):
			self.file_existence_errorlist += errorfile
		else:
			self.file_existence_errorlist.append( errorfile )

		#重複削除
		self.file_existence_errorlist = sorted( set( self.file_existence_errorlist ), key = self.file_existence_errorlist.index )

	def get_existence_error( self ):
		return self.file_existence_errorlist

	def add_read_error( self, errorfile ):
		if isinstance( errorfile, list ):
			self.fileread_errorlist += errorfile
		else:
			self.fileread_errorlist.append( errorfile )

		#重複削除
		self.fileread_errorlist = sorted( set( self.fileread_errorlist ), key = self.fileread_errorlist.index )

	def get_read_error( self ):
		return self.fileread_errorlist

	def set_show_extension( self, mode_bool ):
		self.show_extension = mode_bool

	def get_show_extension( self ):
		return self.show_extension

	def get_base_effects( self ):
		return self.base_effects

	def add_base_effects( self, effect_name, effect_value = True ):
		self.base_effects[ effect_name ] = effect_value

	def get_background_image( self ):
		return self.background_wximage

	def set_ini_path( self, path ):
		self.ini_path = path

	def get_ini_path( self ):
		return self.ini_path

	def set_default_directory( self, directory_path ):
		self.default_directory = directory_path

	def get_default_directory( self ):
		return self.default_directory

	def set_drop_mode( self, dmode ):
		self.drop_mode = dmode

	def get_drop_mode( self ):
		return self.drop_mode



if __name__ == '__main__':

	#初期値設定
	file_list = []
	window_mode = "maximize"
	window_size = []
	window_position = []
	initial_statusbar = False
	bg_colors = []

	reference_values = BaseValue_and_Errorlist()


	#INIの処理
	reference_values.set_ini_path( os.path.splitext( sys.argv[0] )[0] + ".ini" )

	if os.path.isfile( reference_values.get_ini_path() ):
		inisetting = configparser.ConfigParser()
		inisetting.read( reference_values.get_ini_path() )

		#ウィンドウのモード（全画面か通常ウィンドウか）
		if inisetting.has_option( "settings", "window-mode" ):
			window_mode_setting = inisetting.get( "settings", "window-mode" )
			if re.match( "(maximize|window|fullscreen)", window_mode_setting ):
				window_mode = window_mode_setting

		#ウィンドウサイズ（幅x高さ）
		if inisetting.has_option( "settings", "window-size" ):
			window_size_setting = inisetting.get( "settings", "window-size" )
			if window_size_setting != "" and re.match( "^[0-9]+[x][0-9]+", window_size_setting ):
				window_size = window_size_setting.split( "x" )

		#ウィンドウの位置（X,Y）
		if inisetting.has_option( "settings", "window-position" ):
			window_position_setting = inisetting.get( "settings", "window-position" )
			if window_position_setting != "" and re.match( "^[0-9]+[,][0-9]+", window_position_setting ):
				window_position = window_position_setting.split( "," )

		#ステータスバーを起動時から表示させる
		if inisetting.has_option( "settings", "statusbar" ):
			statusbar_setting = inisetting.get( "settings", "statusbar" )
			if statusbar_setting.lower() == "true":
				initial_statusbar = True

		#縦長の画像を2つ並べる場合に、左から右か、右から左かの方向の設定
		if inisetting.has_option( "settings", "page-direction" ):
			direction_setting = inisetting.get( "settings", "page-direction" )
			if direction_setting == "RightToLeft":
				reference_values.set_page_direction( "RightToLeft" )
			else:
				reference_values.set_page_direction( "LeftToRight" )

		#縦長画像だとみなして2つ並べて表示するかどうかを判定する境界線の縦横比
		#現在位置と次の画像の縦横比が両方ともこの数値以下だと、並べて表示する
		if inisetting.has_option( "settings", "borderline-rate" ):
			border_rate_setting = inisetting.get( "settings", "borderline-rate" )
			reference_values.set_borderline_rate( float( border_rate_setting ) )

		#背景色の指定
		if inisetting.has_option( "settings", "background-color-rgb" ):
			bg_colors_setting = inisetting.get( "settings", "background-color-rgb" )
			if re.match( "^[0-9]{1,3}[,][0-9]{1,3}[,][0-9]{1,3}", bg_colors_setting ):
				bg_colors = bg_colors_setting.split( "," )

		#タブにファイルの拡張子も表示する（デフォルトは非表示）
		#引数だと「-show-extension」だけだが、ini中では「= show」なども付ける必要がある
		if inisetting.has_option( "settings", "show-extension" ):
			tab_extension_setting = inisetting.get( "settings", "show-extension" )
			if tab_extension_setting.lower() == "true":
				reference_values.set_show_extension( True )

		#ファイルをドラッグ＆ドロップしたときに、どの位置で開くか
		if inisetting.has_option( "settings", "drop-mode" ):
			dropmode_setting = inisetting.get( "settings", "drop-mode" )
			if dropmode_setting.lower() == "current-right":
				reference_values.set_drop_mode( "current-right" )
			elif dropmode_setting.lower() == "ask":
				reference_values.set_drop_mode( "ask" )

		#＜INI中の効果＞

		if inisetting.has_option( "effects", "rotate" ):
			rotate_setting = inisetting.get( "effects", "rotate" )
			if re.match( "^[0-9]+$", rotate_setting ):
				reference_values.add_base_effects( "rotate", int( rotate_setting ) )

		if inisetting.has_option( "effects", "horizontal-reverse" ):
			horizontal_reverse_setting = inisetting.get( "effects", "horizontal-reverse" )
			if horizontal_reverse_setting.lower() == "true":
				reference_values.add_base_effects( "horizontal-reverse", bool( horizontal_reverse_setting ) )

		if inisetting.has_option( "effects", "vertical-reverse" ):
			vertical_reverse_setting = inisetting.get( "effects", "vertical-reverse" )
			if vertical_reverse_setting.lower() == "true":
				reference_values.add_base_effects( "vertical-reverse", bool( vertical_reverse_setting ) )

		if inisetting.has_option( "effects", "brightness" ):
			brightness_setting = inisetting.get( "effects", "brightness" )
			if re.match( "^[0-9]+$", brightness_setting ):
				reference_values.add_base_effects( "brightness", int( brightness_setting ) )

		if inisetting.has_option( "effects", "contrast" ):
			contrast_setting = inisetting.get( "effects", "contrast" )
			if re.match( "^[0-9]+$", contrast_setting ):
				reference_values.add_base_effects( "contrast",int( contrast_setting ) )


	#引数の処理
	for com_line_arg in sys.argv:

		#画像ファイルを羅列したリストファイルを渡された場合
		if re.match( "^-listfile=", com_line_arg ):
			listfile_path = com_line_arg.lstrip( com_line_arg.split( "=" )[0] + "=" )

			if os.path.exists( listfile_path ) and os.path.isfile( listfile_path ):
				f = open( listfile_path, "r" )

				for list_content in f.readlines():
					#次の行で改行を除去しないとファイル名として正しく扱えない。
					list_content = list_content.rstrip( '\n' )
					list_content = list_content.rstrip( '\r' )

					file_list.append( os.path.abspath( list_content ) )

				f.close()

		elif re.match( "^-window-mode=(maximize|window|fullscreen)", com_line_arg ):
			window_mode = com_line_arg.split( "=" )[1]

		elif re.match( "^-window-size=[0-9]+[x][0-9]+", com_line_arg ):
			temporary_size  = com_line_arg.split( "=" )[1]
			window_size = temporary_size.split( "x" )

		elif re.match( "^-window-position=[0-9]+[,][0-9]+", com_line_arg ):
			temporary_pos  = com_line_arg.split( "=" )[1]
			window_position = temporary_pos.split( "," )

		elif re.match( "^-(show|hide)-statusbar", com_line_arg ):
			if com_line_arg == -show-statusbar:
				initial_statusbar = True
			else:
				initial_statusbar = False

		elif re.match( "^-page-direction=(RightToLeft|LeftToRight)", com_line_arg ):
			temporary_direction = com_line_arg.split( "=" )[1]

			if temporary_direction== "RightToLeft":
				reference_values.set_page_direction( "RightToLeft" )
			else:
				reference_values.set_page_direction( "LeftToRight" )

		elif re.match( "^-borderline-rate=[0-9]+([.][0-9]+)?", com_line_arg ):
			reference_values.set_borderline_rate( float( com_line_arg.split( "=" )[1] ) )

		elif re.match( "^-background-color-rgb=[0-9]{1,3}[,][0-9]{1,3}[,][0-9]{1,3}", com_line_arg ):
			temporary_bgc = com_line_arg.split( "=" )[1]
			bg_colors = temporary_bgc.split( "," )

		elif re.match( "^-effect/(rotate|brightness|contrast)=[0-9]+", com_line_arg ):
			effect_kind = com_line_arg.split( "=" )[0]
			effect_kind = effect_kind.split( "/" )[1]
			effect_value = com_line_arg.split( "=" )[1]
			reference_values.add_base_effects( effect_kind, int( effect_value ) )

		elif re.match( "^-effect/(horizontal-reverse|vertical-reverse)", com_line_arg ):
			effect_kind = com_line_arg.split( "=" )[0]
			effect_kind = effect_kind.split( "/" )[1]
			reference_values.add_base_effects( effect_kind )

		#タブにファイルの拡張子も表示する
		elif re.match( "^-(show|hide)-extension", com_line_arg ):
			if com_line_arg == -show-extension:
				reference_values.set_show_extension( True )
			else:
				reference_values.set_show_extension( False )

		#ドラッグ＆ドロップの対応を設定する
		elif re.match( "^-dragdrop-mode=(right-end|current-right|ask)", com_line_arg ):
			temporary_dragmode  = com_line_arg.split( "=" )[1]

			if temporary_direction== "current-right":
				reference_values.set_drop_mode( "current-right" )
			elif temporary_direction== "ask":
				reference_values.set_drop_mode( "ask" )
			else:
				reference_values.set_drop_mode( "" )

		#何らかのファイルを直接引数として渡された場合
		elif os.path.exists( com_line_arg ) and os.path.isfile( com_line_arg ):

			#もし、テキストファイルだったら、画像ファイルのパスを記したリストファイルだとみなして読み込んでいく。
			if mimetypes.guess_type( com_line_arg )[0] == "text/plain":
				f = open( com_line_arg, "r" )
				relatively_position = 0

				for one_line in f.readlines():
					#次の行で改行を除去しないとファイル名として正しく扱えない。
					one_line = one_line.rstrip( '\n' )
					one_line = one_line.rstrip( '\r' )
					line_abspath = os.path.abspath( one_line )

					if os.path.isfile( line_abspath ) and re.match( "image/", str( mimetypes.guess_type( line_abspath )[0] ) ):
						file_list.append( os.path.abspath( line_abspath ) )

				f.close()

			#引数で渡されたパスが画像ファイルだったら、開くファイルのリストにそのまま追加する。
			elif re.match( "image/", str( mimetypes.guess_type( com_line_arg )[0] ) ):
				file_list.append( os.path.abspath( com_line_arg ) )

		#フォルダが引数の場合
		elif os.path.isdir( com_line_arg ):
			#画像がないフォルダだけを引数として渡された場合、ファイルオープンダイアログが
			#渡されたフォルダから開始するように、デフォルトディレクトリとして登録しておく。
			reference_values.set_default_directory( os.path.abspath( com_line_arg ) )
			
			imagefile_list = detect_imagefiles_under_folder( os.path.abspath( com_line_arg ) )
			if len( imagefile_list ) > 0:
				#画像のmimetypeになっているファイルを最初の1つだけ追加する。
				file_list.append( imagefile_list[0] )

		else:
			reference_values.add_existence_error( com_line_arg )

	#画像のmimetypeになっているファイルのみをまとめる
	image_file_list = []

	for i in file_list:

		if os.path.isfile(i) and re.match( "image/", str( mimetypes.guess_type(i)[0] ) ):
			image_file_list.append(i)

	#引数の処理はここまで


	if window_size == []:
		window_size = ( 1024, 768 )
	else:
		window_size = tuple( [ int( i ) for i in window_size ] )

	frame = extended_frame( None, wx.ID_ANY, "ZetaView", size = window_size )

	if window_mode == "maximize":
		frame.Maximize()
	elif window_mode == "fullscreen":
		frame.ShowFullScreen( True )

	if initial_statusbar:
		frame.statusbar_show_or_hide( "show" )

	if bg_colors != []:
		reference_values.set_bgcolor( [ int(i) for i in bg_colors ] )

	if window_position == []:
		frame.Centre()
	else:
		frame.SetPosition( tuple( [ int(i) for i in window_position ] ) )

	frame.Show()

	frame.add_picture_tab( image_file_list )

	application.MainLoop()
